; File: "syntax-case.scm", Time-stamp: <2005-01-04 10:52:34 feeley>

; Copyright (C) 1998-2005 by Marc Feeley, All Rights Reserved.

; Version 2.0 (October 18, 2004)

; This is an implementation of "syntax-case" for the Gambit-C 4.0
; interpreter (gsi) based on the portable implementation by Hieb and
; Dybvig.

; You can simply load this file into the interpreter with:
;
;   (load "syntax-case")
;
; For faster macro processing compile the file with the compiler.
; You can also rename this file to "gambcext.scm" and put it
; in the Gambit installation directory so that it is loaded every
; time gsi is started.

; Note that macros are only visible in the interpreter and that line
; number information is lost because of the way the expander works
; (error messages will point to the toplevel form that contains the
; error).

(##declare
  (standard-bindings)
  (extended-bindings)
  (not safe)
)

(##namespace ("syntax-case#"))

(##namespace (""
   not boolean? eqv? eq? equal? pair? cons car cdr set-car! set-cdr!
   caar cadr cdar cddr caaar caadr cadar caddr cdaar cdadr cddar cdddr
   caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr cdaaar cdaadr
   cdadar cdaddr cddaar cddadr cdddar cddddr null? list? list length
   append reverse list-ref memq memv member assq assv assoc symbol?
   symbol->string string->symbol number? complex? real? rational?
   integer? exact? inexact? = < > <= >= zero? positive? negative? odd?
   even? max min + * - / abs quotient remainder modulo gcd lcm numerator
   denominator floor ceiling truncate round rationalize exp log sin cos
   tan asin acos atan sqrt expt make-rectangular make-polar real-part
   imag-part magnitude angle exact->inexact inexact->exact number->string
   string->number char? char=? char<? char>? char<=? char>=? char-ci=?
   char-ci<? char-ci>? char-ci<=? char-ci>=? char-alphabetic?
   char-numeric? char-whitespace? char-upper-case? char-lower-case?
   char->integer integer->char char-upcase char-downcase string?
   make-string string string-length string-ref string-set! string=?
   string<? string>? string<=? string>=? string-ci=? string-ci<?
   string-ci>? string-ci<=? string-ci>=? substring string-append vector?
   make-vector vector vector-length vector-ref vector-set! procedure?
   apply map for-each call-with-current-continuation call-with-input-file
   call-with-output-file input-port? output-port? current-input-port
   current-output-port open-input-file open-output-file close-input-port
   close-output-port eof-object? read read-char peek-char write display
   newline write-char

   list->vector vector->list eval gensym void ; non-R4RS procedures

   expand-syntax
   syntax-dispatch
   generate-temporaries
   identifier?
   syntax-error
   syntax-object->datum
   bound-identifier=?
   free-identifier=?
   install-global-transformer
   implicit-identifier

   error
))

;------------------------------------------------------------------------------

; Hooks for Gambit.

(define ##expand-source
  (lambda (src)
    (##sourcify (expand-syntax (##desourcify src)) src)))

(define c#expand-source
  (lambda (src)
    (c#expression->source (expand-syntax (##desourcify src)) src)))

(define eval-hook eval)

(define (error-hook where why what)
  (if (eq? what (void))
      (error why)
      (error why what)))

;;; New symbols are used to generate non-capturing bindings.  If it is
;;; impossible to generate unique symbols, output identifiers during
;;; expansion and either feed the result directly into the compiler or
;;; make another pass to perform alpha substitution.

(define (new-symbol-hook str)
  (gensym (string->symbol str)))

;;; "put-global-definition-hook" should overwrite existing definitions.

(define macro-transformers '())

(define put-global-definition-hook
  (lambda (symbol binding)
    (let ((x (assq symbol macro-transformers)))
      (if x
          (set-cdr! x binding)
          (set! macro-transformers
                (cons (cons symbol binding) macro-transformers))))))

;;; "get-global-definition-hook" should return "#f" if no binding
;;; has been established by "put-global-definition-hook" for the symbol.

(define get-global-definition-hook
  (lambda (symbol)
    (let ((x (assq symbol macro-transformers)))
      (if x
          (cdr x)
          #f))))

;------------------------------------------------------------------------------

;;; compat.ss
;;; Robert Hieb & Kent Dybvig
;;; 92/06/18

;;; This file contains nonstandard help procedures.
;;; They are all present in Chez Scheme, but are easily defined
;;; in any standard Scheme system.
;;; These versions do no error checking.

; In Chez Scheme "(void)" returns an object that is ignored by the
; REP loop.  It is returned whenever a "nonspecified" value is specified
; by the standard.  The following should pick up an appropriate value.

;(define void
;   (let ((void-object (if #f #f)))
;      (lambda () void-object)))

; "andmap" is like "map" except instead of "consing" the results it
; "ands" them, quitting early if #f" is obtained.
; The following does no error checking.

(define andmap
   (lambda (f first . rest)
      (cond
         ((null? rest)
          (let mapf ((l first))
             (or (null? l)
                 (and (f (car l)) (mapf (cdr l))))))
         ((null? (cdr rest))
          (let mapf ((l1 first) (l2 (car rest)))
             (or (null? l1)
                 (and (f (car l1) (car l2)) (mapf (cdr l1) (cdr l2))))))
         (else
          (let mapf ((first first) (rest rest))
             (or (null? first)
                 (and (apply f (car first) (map car rest))
                      (mapf (cdr first) (map cdr rest)))))))))

;------------------------------------------------------------------------------

;;; output.ss
;;; Robert Hieb & Kent Dybvig
;;; 92/06/18

; The output routines can be tailored to feed a specific system or compiler.
; They are set up here to generate the following subset of standard Scheme:

;  <expression> :== <application>
;                |  <variable>
;                |  (set! <variable> <expression>)
;                |  (define <variable> <expression>)
;                |  (lambda (<variable>*) <expression>)
;                |  (lambda <variable> <expression>)
;                |  (lambda (<variable>+ . <variable>) <expression>)
;                |  (letrec (<binding>+) <expression>)
;                |  (if <expression> <expression> <expression>)
;                |  (begin <expression> <expression>)
;                |  (quote <datum>)
; <application> :== (<expression>+)
;     <binding> :== (<variable> <expression>)
;    <variable> :== <symbol>

; Definitions are generated only at top level.

(define build-application
   (lambda (fun-exp arg-exps)
      `(,fun-exp ,@arg-exps)))

(define build-conditional
   (lambda (test-exp then-exp else-exp)
      `(if ,test-exp ,then-exp ,else-exp)))

(define build-lexical-reference (lambda (var) var))

(define build-lexical-assignment
   (lambda (var exp)
      `(set! ,var ,exp)))

(define build-global-reference (lambda (var) var))

(define build-global-assignment
   (lambda (var exp)
      `(set! ,var ,exp)))

(define build-lambda
   (lambda (vars exp)
      `(lambda ,vars ,exp)))

(define build-improper-lambda
   (lambda (vars var exp)
      `(lambda (,@vars . ,var) ,exp)))

(define build-data
   (lambda (exp)
      `(quote ,exp)))

(define build-identifier
   (lambda (id)
      `(quote ,id)))

(define build-sequence
   (lambda (exps)
      (if (null? (cdr exps))
          (car exps)
          `(begin ,(car exps) ,(build-sequence (cdr exps))))))

(define build-letrec
   (lambda (vars val-exps body-exp)
      (if (null? vars)
          body-exp
          `(letrec ,(map list vars val-exps) ,body-exp))))

(define build-global-definition
   (lambda (var val)
      `(define ,var ,val)))

;------------------------------------------------------------------------------

;;; init.ss
;;; Robert Hieb & Kent Dybvig
;;; 92/06/18

; These initializations are done here rather than "expand.ss" so that
; "expand.ss" can be loaded twice (for bootstrapping purposes).

(define expand-syntax #f)
(define syntax-dispatch #f)
(define generate-temporaries #f)
(define identifier? #f)
(define syntax-error #f)
(define syntax-object->datum #f)
(define bound-identifier=? #f)
(define free-identifier=? #f)
(define install-global-transformer #f)
(define implicit-identifier #f)

;------------------------------------------------------------------------------

; From file "expand.bb".

(begin
  ((lambda ()
     (letrec ((:-d.lambda-var-list1313
                (lambda (:-d.vars1574)
                  ((letrec ((:-d.lvl1575
                              (lambda (:-d.vars1576 :-d.ls1577)
                                (if (pair? :-d.vars1576)
                                  (:-d.lvl1575
                                    (cdr :-d.vars1576)
                                    (cons (car :-d.vars1576) :-d.ls1577))
                                  (if (:-d.id?1356 :-d.vars1576)
                                    (cons :-d.vars1576 :-d.ls1577)
                                    (if (null? :-d.vars1576)
                                      :-d.ls1577
                                      (if (:-d.syntax-object?1373 :-d.vars1576)
                                        (:-d.lvl1575
                                          (:-d.unwrap1338 :-d.vars1576)
                                          :-d.ls1577)
                                        (cons :-d.vars1576 :-d.ls1577))))))))
                     :-d.lvl1575)
                   :-d.vars1574
                   '())))
              (:-d.gen-var1314
                (lambda (:-d.id1578)
                  (:-d.gen-sym1315 (:-d.id-sym-name1355 :-d.id1578))))
              (:-d.gen-sym1315
                (lambda (:-d.sym1579)
                  (new-symbol-hook (symbol->string :-d.sym1579))))
              (:-d.strip1316
                (lambda (:-d.x1580)
                  (if (:-d.syntax-object?1373 :-d.x1580)
                    (:-d.strip1316
                      (:-d.syntax-object-expression1371 :-d.x1580))
                    (if (pair? :-d.x1580)
                      ((lambda (:-d.a1583 :-d.d1584)
                         (if (if (eq? :-d.a1583 (car :-d.x1580))
                               (eq? :-d.d1584 (cdr :-d.x1580))
                               '#f)
                           :-d.x1580
                           (cons :-d.a1583 :-d.d1584)))
                       (:-d.strip1316 (car :-d.x1580))
                       (:-d.strip1316 (cdr :-d.x1580)))
                      (if (vector? :-d.x1580)
                        ((lambda (:-d.old1581)
                           ((lambda (:-d.new1582)
                              (if (andmap eq? :-d.old1581 :-d.new1582)
                                :-d.x1580
                                (list->vector :-d.new1582)))
                            (map :-d.strip1316 :-d.old1581)))
                         (vector->list :-d.x1580))
                        :-d.x1580)))))
              (:-d.regen1317
                (lambda (:-d.x1585)
                  ((lambda (:-d.x1586)
                     (if (memv :-d.x1586 '(ref))
                       (build-lexical-reference (cadr :-d.x1585))
                       ((lambda (:-d.x1587)
                          (if (memv :-d.x1587 '(primitive))
                            (build-global-reference (cadr :-d.x1585))
                            ((lambda (:-d.x1588)
                               (if (memv :-d.x1588 '(id))
                                 (build-identifier (cadr :-d.x1585))
                                 ((lambda (:-d.x1589)
                                    (if (memv :-d.x1589 '(quote))
                                      (build-data (cadr :-d.x1585))
                                      ((lambda (:-d.x1590)
                                         (if (memv :-d.x1590 '(lambda))
                                           (build-lambda
                                             (cadr :-d.x1585)
                                             (:-d.regen1317 (caddr :-d.x1585)))
                                           (begin
                                             :-d.x1590
                                             (build-application
                                               (build-global-reference
                                                 (car :-d.x1585))
                                               (map :-d.regen1317
                                                    (cdr :-d.x1585))))))
                                       :-d.x1589)))
                                  :-d.x1588)))
                             :-d.x1587)))
                        :-d.x1586)))
                   (car :-d.x1585))))
              (:-d.gen-vector1318
                (lambda (:-d.x1591)
                  (if (eq? (car :-d.x1591) 'list)
                    (cons 'vector (cdr :-d.x1591))
                    (if (eq? (car :-d.x1591) 'quote)
                      (list 'quote (list->vector (cadr :-d.x1591)))
                      (list 'list->vector :-d.x1591)))))
              (:-d.gen-append1319
                (lambda (:-d.x1592 :-d.y1593)
                  (if (equal? :-d.y1593 ''())
                    :-d.x1592
                    (list 'append :-d.x1592 :-d.y1593))))
              (:-d.gen-cons1320
                (lambda (:-d.x1594 :-d.y1595)
                  (if (eq? (car :-d.y1595) 'list)
                    (cons 'list (cons :-d.x1594 (cdr :-d.y1595)))
                    (if (if (eq? (car :-d.x1594) 'quote)
                          (eq? (car :-d.y1595) 'quote)
                          '#f)
                      (list 'quote (cons (cadr :-d.x1594) (cadr :-d.y1595)))
                      (if (equal? :-d.y1595 ''())
                        (list 'list :-d.x1594)
                        (list 'cons :-d.x1594 :-d.y1595))))))
              (:-d.gen-map1321
                (lambda (:-d.e1596 :-d.map-env1597)
                  ((lambda (:-d.formals1599 :-d.actuals1600)
                     (if (eq? (car :-d.e1596) 'ref)
                       (car :-d.actuals1600)
                       (if (andmap (lambda (:-d.x1603)
                                     (if (eq? (car :-d.x1603) 'ref)
                                       (memq (cadr :-d.x1603) :-d.formals1599)
                                       '#f))
                                   (cdr :-d.e1596))
                         (cons 'map
                               (cons (list 'primitive (car :-d.e1596))
                                     (map ((lambda (:-d.r1601)
                                             (lambda (:-d.x1602)
                                               (cdr (assq (cadr :-d.x1602)
                                                          :-d.r1601))))
                                           (map cons
                                                :-d.formals1599
                                                :-d.actuals1600))
                                          (cdr :-d.e1596))))
                         (cons 'map
                               (cons (list 'lambda :-d.formals1599 :-d.e1596)
                                     :-d.actuals1600)))))
                   (map cdr :-d.map-env1597)
                   (map (lambda (:-d.x1598) (list 'ref (car :-d.x1598)))
                        :-d.map-env1597))))
              (:-d.gen-ref1322
                (lambda (:-d.var1604 :-d.level1605 :-d.maps1606 :-d.k1607)
                  (if (= :-d.level1605 '0)
                    (:-d.k1607 :-d.var1604 :-d.maps1606)
                    (:-d.gen-ref1322
                      :-d.var1604
                      (- :-d.level1605 '1)
                      (cdr :-d.maps1606)
                      (lambda (:-d.outer-var1608 :-d.outer-maps1609)
                        ((lambda (:-d.b1610)
                           (if :-d.b1610
                             (:-d.k1607 (cdr :-d.b1610) :-d.maps1606)
                             ((lambda (:-d.inner-var1611)
                                (:-d.k1607
                                  :-d.inner-var1611
                                  (cons (cons (cons :-d.outer-var1608
                                                    :-d.inner-var1611)
                                              (car :-d.maps1606))
                                        :-d.outer-maps1609)))
                              (:-d.gen-sym1315 :-d.var1604))))
                         (assq :-d.outer-var1608 (car :-d.maps1606))))))))
              (:-d.chi-syntax1323
                (lambda (:-d.exp1612 :-d.r1613 :-d.w1614)
                  ((letrec ((:-d.gen1617
                              (lambda (:-d.e1618 :-d.maps1619 :-d.k1620)
                                (if (:-d.id?1356 :-d.e1618)
                                  ((lambda (:-d.n1654)
                                     ((lambda (:-d.b1655)
                                        (if (eq? (:-d.binding-type1366
                                                   :-d.b1655)
                                                 'syntax)
                                          ((lambda (:-d.level1656)
                                             (if (< (length :-d.maps1619)
                                                    :-d.level1656)
                                               (syntax-error
                                                 :-d.exp1612
                                                 '"missing ellipsis in")
                                               (:-d.gen-ref1322
                                                 :-d.n1654
                                                 :-d.level1656
                                                 :-d.maps1619
                                                 (lambda (:-d.x1657
                                                          :-d.maps1658)
                                                   (:-d.k1620
                                                     (list 'ref :-d.x1657)
                                                     :-d.maps1658)))))
                                           (:-d.binding-value1365 :-d.b1655))
                                          (:-d.k1620
                                            (list 'id
                                                  (:-d.wrap1337
                                                    :-d.e1618
                                                    :-d.w1614))
                                            :-d.maps1619)))
                                      (:-d.lookup1358
                                        :-d.n1654
                                        :-d.e1618
                                        :-d.r1613)))
                                   (:-d.id-var-name1344 :-d.e1618 :-d.w1614))
                                  ((lambda (:-d.x1621)
                                     ((lambda (:-d.dorest1650)
                                        ((lambda (:-d.y1651)
                                           (if :-d.y1651
                                             ((lambda (:-d.dots11652
                                                       :-d.dots21653)
                                                (if (if (:-d.id?1356
                                                          :-d.dots11652)
                                                      (if (:-d.id?1356
                                                            :-d.dots21653)
                                                        (if (eq? (:-d.id-var-name1344
                                                                   :-d.dots11652
                                                                   :-d.w1614)
                                                                 '...)
                                                          (eq? (:-d.id-var-name1344
                                                                 :-d.dots21653
                                                                 :-d.w1614)
                                                               '...)
                                                          '#f)
                                                        '#f)
                                                      '#f)
                                                  (:-d.k1620
                                                    (list 'id
                                                          (:-d.wrap1337
                                                            :-d.dots11652
                                                            :-d.w1614))
                                                    :-d.maps1619)
                                                  (:-d.dorest1650)))
                                              (car :-d.y1651)
                                              (cadr :-d.y1651))
                                             (:-d.dorest1650)))
                                         (syntax-dispatch
                                           :-d.x1621
                                           '(pair (any) pair (any) atom)
                                           '#())))
                                      (lambda ()
                                        ((lambda (:-d.x1622)
                                           ((lambda (:-d.dorest1641)
                                              ((lambda (:-d.y1642)
                                                 (if :-d.y1642
                                                   ((lambda (:-d.x1643
                                                             :-d.dots1644
                                                             :-d.y1645)
                                                      (if (if (:-d.id?1356
                                                                :-d.dots1644)
                                                            (eq? (:-d.id-var-name1344
                                                                   :-d.dots1644
                                                                   :-d.w1614)
                                                                 '...)
                                                            '#f)
                                                        (:-d.gen1617
                                                          :-d.y1645
                                                          :-d.maps1619
                                                          (lambda (:-d.y1646
                                                                   :-d.maps1647)
                                                            (:-d.gen1617
                                                              :-d.x1643
                                                              (cons '()
                                                                    :-d.maps1647)
                                                              (lambda (:-d.x1648
                                                                       :-d.maps1649)
                                                                (if (null? (car :-d.maps1649))
                                                                  (syntax-error
                                                                    :-d.exp1612
                                                                    '"extra ellipsis in")
                                                                  (:-d.k1620
                                                                    (:-d.gen-append1319
                                                                      (:-d.gen-map1321
                                                                        :-d.x1648
                                                                        (car :-d.maps1649))
                                                                      :-d.y1646)
                                                                    (cdr :-d.maps1649)))))))
                                                        (:-d.dorest1641)))
                                                    (car :-d.y1642)
                                                    (cadr :-d.y1642)
                                                    (caddr :-d.y1642))
                                                   (:-d.dorest1641)))
                                               (syntax-dispatch
                                                 :-d.x1622
                                                 '(pair (any) pair (any) any)
                                                 '#())))
                                            (lambda ()
                                              ((lambda (:-d.x1623)
                                                 ((lambda (:-d.y1624)
                                                    (if :-d.y1624
                                                      ((lambda (:-d.x1635
                                                                :-d.y1636)
                                                         (:-d.gen1617
                                                           :-d.x1635
                                                           :-d.maps1619
                                                           (lambda (:-d.x1637
                                                                    :-d.maps1638)
                                                             (:-d.gen1617
                                                               :-d.y1636
                                                               :-d.maps1638
                                                               (lambda (:-d.y1639
                                                                        :-d.maps1640)
                                                                 (:-d.k1620
                                                                   (:-d.gen-cons1320
                                                                     :-d.x1637
                                                                     :-d.y1639)
                                                                   :-d.maps1640))))))
                                                       (car :-d.y1624)
                                                       (cadr :-d.y1624))
                                                      ((lambda (:-d.x1625)
                                                         ((lambda (:-d.y1626)
                                                            (if :-d.y1626
                                                              ((lambda (:-d.e11630
                                                                        :-d.e21631)
                                                                 (:-d.gen1617
                                                                   (cons :-d.e11630
                                                                         :-d.e21631)
                                                                   :-d.maps1619
                                                                   (lambda (:-d.e1632
                                                                            :-d.maps1633)
                                                                     (:-d.k1620
                                                                       (:-d.gen-vector1318
                                                                         :-d.e1632)
                                                                       :-d.maps1633))))
                                                               (car :-d.y1626)
                                                               (cadr :-d.y1626))
                                                              ((lambda (:-d.x1627)
                                                                 ((lambda (:-d.y1628)
                                                                    (if :-d.y1628
                                                                      ((lambda (:-d._1629)
                                                                         (:-d.k1620
                                                                           (list 'quote
                                                                                 (:-d.wrap1337
                                                                                   :-d.e1618
                                                                                   :-d.w1614))
                                                                           :-d.maps1619))
                                                                       (car :-d.y1628))
                                                                      (syntax-error
                                                                        :-d.x1627)))
                                                                  (syntax-dispatch
                                                                    :-d.x1627
                                                                    '(any)
                                                                    '#())))
                                                               :-d.x1625)))
                                                          (syntax-dispatch
                                                            :-d.x1625
                                                            '(vector pair
                                                                     (any)
                                                                     each
                                                                     any)
                                                            '#())))
                                                       :-d.x1623)))
                                                  (syntax-dispatch
                                                    :-d.x1623
                                                    '(pair (any) any)
                                                    '#())))
                                               :-d.x1622))))
                                         :-d.x1621))))
                                   :-d.e1618)))))
                     :-d.gen1617)
                   :-d.exp1612
                   '()
                   (lambda (:-d.e1615 :-d.maps1616)
                     (:-d.regen1317 :-d.e1615)))))
              (:-d.chi-syntax-definition1324
                (lambda (:-d.e1659 :-d.w1660)
                  ((lambda (:-d.x1661)
                     ((lambda (:-d.dorest1662)
                        ((lambda (:-d.y1663)
                           (if :-d.y1663
                             ((lambda (:-d._1664 :-d.name1665 :-d.val1666)
                                (if (:-d.id?1356 :-d.name1665)
                                  (list :-d.name1665 :-d.val1666)
                                  (:-d.dorest1662)))
                              (car :-d.y1663)
                              (cadr :-d.y1663)
                              (caddr :-d.y1663))
                             (:-d.dorest1662)))
                         (syntax-dispatch
                           :-d.x1661
                           '(pair (any) pair (any) pair (any) atom)
                           '#())))
                      (lambda () (syntax-error :-d.x1661))))
                   (:-d.wrap1337 :-d.e1659 :-d.w1660))))
              (:-d.chi-definition1325
                (lambda (:-d.e1667 :-d.w1668)
                  ((lambda (:-d.ee1669)
                     ((lambda (:-d.x1670)
                        ((lambda (:-d.dorest1690)
                           ((lambda (:-d.y1691)
                              (if :-d.y1691
                                (apply (lambda (:-d._1692
                                                :-d.name1693
                                                :-d.arg11694
                                                :-d.arg21695
                                                :-d.e11696
                                                :-d.e21697)
                                         (if (if (:-d.id?1356 :-d.name1693)
                                               (if (:-d.valid-bound-ids?1340
                                                     (:-d.lambda-var-list1313
                                                       :-d.arg11694))
                                                 (:-d.valid-bound-ids?1340
                                                   (:-d.lambda-var-list1313
                                                     :-d.arg21695))
                                                 '#f)
                                               '#f)
                                           (list :-d.name1693
                                                 (list 'lambda
                                                       :-d.arg11694
                                                       (cons 'lambda
                                                             (cons :-d.arg21695
                                                                   (cons :-d.e11696
                                                                         :-d.e21697)))))
                                           (:-d.dorest1690)))
                                       :-d.y1691)
                                (:-d.dorest1690)))
                            (syntax-dispatch
                              :-d.x1670
                              '(pair (any)
                                     pair
                                     (pair (pair (any) any) any)
                                     pair
                                     (any)
                                     each
                                     any)
                              '#())))
                         (lambda ()
                           ((lambda (:-d.x1671)
                              ((lambda (:-d.dorest1682)
                                 ((lambda (:-d.y1683)
                                    (if :-d.y1683
                                      (apply (lambda (:-d._1684
                                                      :-d.name1685
                                                      :-d.args1686
                                                      :-d.e11687
                                                      :-d.e21688)
                                               (if (if (:-d.id?1356
                                                         :-d.name1685)
                                                     (:-d.valid-bound-ids?1340
                                                       (:-d.lambda-var-list1313
                                                         :-d.args1686))
                                                     '#f)
                                                 (list :-d.name1685
                                                       (cons 'lambda
                                                             (cons :-d.args1686
                                                                   (cons :-d.e11687
                                                                         :-d.e21688))))
                                                 (:-d.dorest1682)))
                                             :-d.y1683)
                                      (:-d.dorest1682)))
                                  (syntax-dispatch
                                    :-d.x1671
                                    '(pair (any)
                                           pair
                                           (pair (any) any)
                                           pair
                                           (any)
                                           each
                                           any)
                                    '#())))
                               (lambda ()
                                 ((lambda (:-d.x1672)
                                    ((lambda (:-d.y1673)
                                       (if :-d.y1673
                                         ((lambda (:-d._1679
                                                   :-d.name1680
                                                   :-d.val1681)
                                            (list :-d.name1680 :-d.val1681))
                                          (car :-d.y1673)
                                          (cadr :-d.y1673)
                                          (caddr :-d.y1673))
                                         ((lambda (:-d.x1674)
                                            ((lambda (:-d.dorest1675)
                                               ((lambda (:-d.y1676)
                                                  (if :-d.y1676
                                                    ((lambda (:-d._1677
                                                              :-d.name1678)
                                                       (if (:-d.id?1356
                                                             :-d.name1678)
                                                         (list :-d.name1678
                                                               (list 'void))
                                                         (:-d.dorest1675)))
                                                     (car :-d.y1676)
                                                     (cadr :-d.y1676))
                                                    (:-d.dorest1675)))
                                                (syntax-dispatch
                                                  :-d.x1674
                                                  '(pair (any) pair (any) atom)
                                                  '#())))
                                             (lambda ()
                                               (syntax-error :-d.x1674))))
                                          :-d.x1672)))
                                     (syntax-dispatch
                                       :-d.x1672
                                       '(pair (any) pair (any) pair (any) atom)
                                       '#())))
                                  :-d.x1671))))
                            :-d.x1670))))
                      :-d.ee1669))
                   (:-d.wrap1337 :-d.e1667 :-d.w1668))))
              (:-d.chi-sequence1326
                (lambda (:-d.e1699 :-d.w1700)
                  ((lambda (:-d.x1701)
                     ((lambda (:-d.y1702)
                        (if :-d.y1702
                          ((lambda (:-d._1703 :-d.e1704) :-d.e1704)
                           (car :-d.y1702)
                           (cadr :-d.y1702))
                          (syntax-error :-d.x1701)))
                      (syntax-dispatch :-d.x1701 '(pair (any) each any) '#())))
                   (:-d.wrap1337 :-d.e1699 :-d.w1700))))
              (:-d.chi-macro-def1327
                (lambda (:-d.def1706 :-d.r1707 :-d.w1708)
                  (eval-hook
                    (:-d.chi1335 :-d.def1706 :-d.null-env1362 :-d.w1708))))
              (:-d.chi-local-syntax1328
                (lambda (:-d.e1709 :-d.r1710 :-d.w1711)
                  ((lambda (:-d.x1712)
                     ((lambda (:-d.dorest1716)
                        ((lambda (:-d.y1717)
                           (if :-d.y1717
                             (apply (lambda (:-d.who1718
                                             :-d.var1719
                                             :-d.val1720
                                             :-d.e11721
                                             :-d.e21722)
                                      (if (:-d.valid-bound-ids?1340
                                            :-d.var1719)
                                        ((lambda (:-d.new-vars1724)
                                           ((lambda (:-d.new-w1726)
                                              (:-d.chi-body1329
                                                (cons :-d.e11721 :-d.e21722)
                                                :-d.e1709
                                                (:-d.extend-macro-env1361
                                                  :-d.new-vars1724
                                                  ((lambda (:-d.w1727)
                                                     (map (lambda (:-d.x1729)
                                                            (:-d.chi-macro-def1327
                                                              :-d.x1729
                                                              :-d.r1710
                                                              :-d.w1727))
                                                          :-d.val1720))
                                                   (if (:-d.free-id=?1342
                                                         :-d.who1718
                                                         'letrec-syntax)
                                                     :-d.new-w1726
                                                     :-d.w1711))
                                                  :-d.r1710)
                                                :-d.new-w1726))
                                            (:-d.make-binding-wrap1350
                                              :-d.var1719
                                              :-d.new-vars1724
                                              :-d.w1711)))
                                         (map :-d.gen-var1314 :-d.var1719))
                                        (:-d.dorest1716)))
                                    :-d.y1717)
                             (:-d.dorest1716)))
                         (syntax-dispatch
                           :-d.x1712
                           '(pair (any)
                                  pair
                                  (each pair (any) pair (any) atom)
                                  pair
                                  (any)
                                  each
                                  any)
                           '#())))
                      (lambda ()
                        ((lambda (:-d.x1713)
                           ((lambda (:-d.y1714)
                              (if :-d.y1714
                                ((lambda (:-d._1715)
                                   (syntax-error
                                     (:-d.wrap1337 :-d.e1709 :-d.w1711)))
                                 (car :-d.y1714))
                                (syntax-error :-d.x1713)))
                            (syntax-dispatch :-d.x1713 '(any) '#())))
                         :-d.x1712))))
                   :-d.e1709)))
              (:-d.chi-body1329
                (lambda (:-d.body1732 :-d.source1733 :-d.r1734 :-d.w1735)
                  (if (null? (cdr :-d.body1732))
                    (:-d.chi1335 (car :-d.body1732) :-d.r1734 :-d.w1735)
                    ((letrec ((:-d.parse11737
                                (lambda (:-d.body1738
                                         :-d.var-ids1739
                                         :-d.var-vals1740
                                         :-d.macro-ids1741
                                         :-d.macro-vals1742)
                                  (if (null? :-d.body1738)
                                    (syntax-error
                                      (:-d.wrap1337 :-d.source1733 :-d.w1735)
                                      '"no expressions in body")
                                    ((letrec ((:-d.parse21743
                                                (lambda (:-d.e1744)
                                                  ((lambda (:-d.b1745)
                                                     ((lambda (:-d.x1746)
                                                        (if (memv :-d.x1746
                                                                  '(macro))
                                                          (:-d.parse21743
                                                            (:-d.chi-macro1333
                                                              (:-d.binding-value1365
                                                                :-d.b1745)
                                                              :-d.e1744
                                                              :-d.r1734
                                                              :-d.empty-wrap1354
                                                              (lambda (:-d.e1757
                                                                       :-d.r1758
                                                                       :-d.w1759)
                                                                (:-d.wrap1337
                                                                  :-d.e1757
                                                                  :-d.w1759))))
                                                          ((lambda (:-d.x1747)
                                                             (if (memv :-d.x1747
                                                                       '(definition))
                                                               (:-d.parse11737
                                                                 (cdr :-d.body1738)
                                                                 (cons (cadr :-d.b1745)
                                                                       :-d.var-ids1739)
                                                                 (cons (caddr :-d.b1745)
                                                                       :-d.var-vals1740)
                                                                 :-d.macro-ids1741
                                                                 :-d.macro-vals1742)
                                                               ((lambda (:-d.x1748)
                                                                  (if (memv :-d.x1748
                                                                            '(syntax-definition))
                                                                    (:-d.parse11737
                                                                      (cdr :-d.body1738)
                                                                      :-d.var-ids1739
                                                                      :-d.var-vals1740
                                                                      (cons (cadr :-d.b1745)
                                                                            :-d.macro-ids1741)
                                                                      (cons (caddr :-d.b1745)
                                                                            :-d.macro-vals1742))
                                                                    ((lambda (:-d.x1749)
                                                                       (if (memv :-d.x1749
                                                                                 '(sequence))
                                                                         (:-d.parse11737
                                                                           (append (cdr :-d.b1745)
                                                                                   (cdr :-d.body1738))
                                                                           :-d.var-ids1739
                                                                           :-d.var-vals1740
                                                                           :-d.macro-ids1741
                                                                           :-d.macro-vals1742)
                                                                         (begin
                                                                           :-d.x1749
                                                                           (if (:-d.valid-bound-ids?1340
                                                                                 (append :-d.var-ids1739
                                                                                         :-d.macro-ids1741))
                                                                             ((lambda (:-d.new-var-names1750
                                                                                       :-d.new-macro-names1751)
                                                                                ((lambda (:-d.w1752)
                                                                                   ((lambda (:-d.r1754)
                                                                                      (build-letrec
                                                                                        :-d.new-var-names1750
                                                                                        (map (lambda (:-d.x1756)
                                                                                               (:-d.chi1335
                                                                                                 :-d.x1756
                                                                                                 :-d.r1754
                                                                                                 :-d.w1752))
                                                                                             :-d.var-vals1740)
                                                                                        (build-sequence
                                                                                          (map (lambda (:-d.x1755)
                                                                                                 (:-d.chi1335
                                                                                                   :-d.x1755
                                                                                                   :-d.r1754
                                                                                                   :-d.w1752))
                                                                                               :-d.body1738))))
                                                                                    (:-d.extend-macro-env1361
                                                                                      :-d.new-macro-names1751
                                                                                      (map (lambda (:-d.x1753)
                                                                                             (:-d.chi-macro-def1327
                                                                                               :-d.x1753
                                                                                               :-d.r1734
                                                                                               :-d.w1752))
                                                                                           :-d.macro-vals1742)
                                                                                      (:-d.extend-var-env1360
                                                                                        :-d.new-var-names1750
                                                                                        :-d.r1734))))
                                                                                 (:-d.make-binding-wrap1350
                                                                                   (append :-d.macro-ids1741
                                                                                           :-d.var-ids1739)
                                                                                   (append :-d.new-macro-names1751
                                                                                           :-d.new-var-names1750)
                                                                                   :-d.empty-wrap1354)))
                                                                              (map :-d.gen-var1314
                                                                                   :-d.var-ids1739)
                                                                              (map :-d.gen-var1314
                                                                                   :-d.macro-ids1741))
                                                                             (syntax-error
                                                                               (:-d.wrap1337
                                                                                 :-d.source1733
                                                                                 :-d.w1735)
                                                                               '"invalid identifier")))))
                                                                     :-d.x1748)))
                                                                :-d.x1747)))
                                                           :-d.x1746)))
                                                      (car :-d.b1745)))
                                                   (:-d.syntax-type1330
                                                     :-d.e1744
                                                     :-d.r1734
                                                     :-d.empty-wrap1354)))))
                                       :-d.parse21743)
                                     (car :-d.body1738))))))
                       :-d.parse11737)
                     (map (lambda (:-d.x1736)
                            (:-d.wrap1337 :-d.x1736 :-d.w1735))
                          :-d.body1732)
                     '()
                     '()
                     '()
                     '()))))
              (:-d.syntax-type1330
                (lambda (:-d.e1760 :-d.r1761 :-d.w1762)
                  (if (:-d.syntax-object?1373 :-d.e1760)
                    (:-d.syntax-type1330
                      (:-d.syntax-object-expression1371 :-d.e1760)
                      :-d.r1761
                      (:-d.join-wraps1348
                        (:-d.syntax-object-wrap1370 :-d.e1760)
                        :-d.w1762))
                    (if (if (pair? :-d.e1760)
                          (identifier? (car :-d.e1760))
                          '#f)
                      ((lambda (:-d.n1763)
                         ((lambda (:-d.b1764)
                            ((lambda (:-d.x1765)
                               (if (memv :-d.x1765 '(special))
                                 ((lambda (:-d.x1766)
                                    (if (memv :-d.x1766 '(define))
                                      (cons 'definition
                                            (:-d.chi-definition1325
                                              :-d.e1760
                                              :-d.w1762))
                                      ((lambda (:-d.x1767)
                                         (if (memv :-d.x1767 '(define-syntax))
                                           (cons 'syntax-definition
                                                 (:-d.chi-syntax-definition1324
                                                   :-d.e1760
                                                   :-d.w1762))
                                           (if (memv :-d.x1767 '(begin))
                                             (cons 'sequence
                                                   (:-d.chi-sequence1326
                                                     :-d.e1760
                                                     :-d.w1762))
                                             (void))))
                                       :-d.x1766)))
                                  :-d.n1763)
                                 (begin :-d.x1765 :-d.b1764)))
                             (:-d.binding-type1366 :-d.b1764)))
                          (:-d.lookup1358
                            :-d.n1763
                            (car :-d.e1760)
                            :-d.r1761)))
                       (:-d.id-var-name1344 (car :-d.e1760) :-d.w1762))
                      '(other)))))
              (:-d.chi-args1331
                (lambda (:-d.args1768
                         :-d.r1769
                         :-d.w1770
                         :-d.source1771
                         :-d.source-w1772)
                  (if (pair? :-d.args1768)
                    (cons (:-d.chi1335 (car :-d.args1768) :-d.r1769 :-d.w1770)
                          (:-d.chi-args1331
                            (cdr :-d.args1768)
                            :-d.r1769
                            :-d.w1770
                            :-d.source1771
                            :-d.source-w1772))
                    (if (null? :-d.args1768)
                      '()
                      (if (:-d.syntax-object?1373 :-d.args1768)
                        (:-d.chi-args1331
                          (:-d.syntax-object-expression1371 :-d.args1768)
                          :-d.r1769
                          (:-d.join-wraps1348
                            :-d.w1770
                            (:-d.syntax-object-wrap1370 :-d.args1768))
                          :-d.source1771
                          :-d.source-w1772)
                        (syntax-error
                          (:-d.wrap1337 :-d.source1771 :-d.source-w1772)))))))
              (:-d.chi-ref1332
                (lambda (:-d.e1773 :-d.name1774 :-d.binding1775 :-d.w1776)
                  ((lambda (:-d.x1777)
                     (if (memv :-d.x1777 '(lexical))
                       (build-lexical-reference :-d.name1774)
                       ((lambda (:-d.x1778)
                          (if (memv :-d.x1778 '(global global-unbound))
                            (build-global-reference :-d.name1774)
                            (begin
                              :-d.x1778
                              (:-d.id-error1368
                                (:-d.wrap1337 :-d.e1773 :-d.w1776)))))
                        :-d.x1777)))
                   (:-d.binding-type1366 :-d.binding1775))))
              (:-d.chi-macro1333
                (lambda (:-d.p1779 :-d.e1780 :-d.r1781 :-d.w1782 :-d.k1783)
                  ((lambda (:-d.mw1784)
                     (:-d.k1783
                       (:-d.p1779
                         (:-d.wrap1337
                           :-d.e1780
                           (:-d.join-wraps1348 :-d.mw1784 :-d.w1782)))
                       :-d.r1781
                       :-d.mw1784))
                   (:-d.new-mark-wrap1351))))
              (:-d.chi-pair1334
                (lambda (:-d.e1785 :-d.r1786 :-d.w1787 :-d.k1788)
                  ((lambda (:-d.first1789 :-d.rest1790)
                     (if (:-d.id?1356 :-d.first1789)
                       ((lambda (:-d.n1791)
                          ((lambda (:-d.b1792)
                             ((lambda (:-d.x1793)
                                (if (memv :-d.x1793 '(core))
                                  ((:-d.binding-value1365 :-d.b1792)
                                   :-d.e1785
                                   :-d.r1786
                                   :-d.w1787)
                                  ((lambda (:-d.x1794)
                                     (if (memv :-d.x1794 '(macro))
                                       (:-d.chi-macro1333
                                         (:-d.binding-value1365 :-d.b1792)
                                         :-d.e1785
                                         :-d.r1786
                                         :-d.w1787
                                         :-d.k1788)
                                       ((lambda (:-d.x1795)
                                          (if (memv :-d.x1795 '(special))
                                            ((:-d.binding-value1365 :-d.b1792)
                                             :-d.e1785
                                             :-d.r1786
                                             :-d.w1787
                                             :-d.k1788)
                                            (begin
                                              :-d.x1795
                                              (build-application
                                                (:-d.chi-ref1332
                                                  :-d.first1789
                                                  :-d.n1791
                                                  :-d.b1792
                                                  :-d.w1787)
                                                (:-d.chi-args1331
                                                  :-d.rest1790
                                                  :-d.r1786
                                                  :-d.w1787
                                                  :-d.e1785
                                                  :-d.w1787)))))
                                        :-d.x1794)))
                                   :-d.x1793)))
                              (:-d.binding-type1366 :-d.b1792)))
                           (:-d.lookup1358 :-d.n1791 :-d.first1789 :-d.r1786)))
                        (:-d.id-var-name1344 :-d.first1789 :-d.w1787))
                       (build-application
                         (:-d.chi1335 :-d.first1789 :-d.r1786 :-d.w1787)
                         (:-d.chi-args1331
                           :-d.rest1790
                           :-d.r1786
                           :-d.w1787
                           :-d.e1785
                           :-d.w1787))))
                   (car :-d.e1785)
                   (cdr :-d.e1785))))
              (:-d.chi1335
                (lambda (:-d.e1796 :-d.r1797 :-d.w1798)
                  (if (symbol? :-d.e1796)
                    ((lambda (:-d.n1802)
                       (:-d.chi-ref1332
                         :-d.e1796
                         :-d.n1802
                         (:-d.lookup1358 :-d.n1802 :-d.e1796 :-d.r1797)
                         :-d.w1798))
                     (:-d.id-var-name1344 :-d.e1796 :-d.w1798))
                    (if (pair? :-d.e1796)
                      (:-d.chi-pair1334
                        :-d.e1796
                        :-d.r1797
                        :-d.w1798
                        :-d.chi1335)
                      (if (:-d.syntax-object?1373 :-d.e1796)
                        (:-d.chi1335
                          (:-d.syntax-object-expression1371 :-d.e1796)
                          :-d.r1797
                          (:-d.join-wraps1348
                            :-d.w1798
                            (:-d.syntax-object-wrap1370 :-d.e1796)))
                        (if ((lambda (:-d.t1799)
                               (if :-d.t1799
                                 :-d.t1799
                                 ((lambda (:-d.t1800)
                                    (if :-d.t1800
                                      :-d.t1800
                                      ((lambda (:-d.t1801)
                                         (if :-d.t1801
                                           :-d.t1801
                                           (char? :-d.e1796)))
                                       (string? :-d.e1796))))
                                  (number? :-d.e1796))))
                             (boolean? :-d.e1796))
                          (build-data :-d.e1796)
                          (syntax-error
                            (:-d.wrap1337 :-d.e1796 :-d.w1798))))))))
              (:-d.chi-top1336
                (lambda (:-d.e1803 :-d.r1804 :-d.w1805)
                  (if (pair? :-d.e1803)
                    (:-d.chi-pair1334
                      :-d.e1803
                      :-d.r1804
                      :-d.w1805
                      :-d.chi-top1336)
                    (if (:-d.syntax-object?1373 :-d.e1803)
                      (:-d.chi-top1336
                        (:-d.syntax-object-expression1371 :-d.e1803)
                        :-d.r1804
                        (:-d.join-wraps1348
                          :-d.w1805
                          (:-d.syntax-object-wrap1370 :-d.e1803)))
                      (:-d.chi1335 :-d.e1803 :-d.r1804 :-d.w1805)))))
              (:-d.wrap1337
                (lambda (:-d.x1806 :-d.w1807)
                  (if (null? :-d.w1807)
                    :-d.x1806
                    (if (:-d.syntax-object?1373 :-d.x1806)
                      (:-d.make-syntax-object1372
                        (:-d.syntax-object-expression1371 :-d.x1806)
                        (:-d.join-wraps1348
                          :-d.w1807
                          (:-d.syntax-object-wrap1370 :-d.x1806)))
                      (if (null? :-d.x1806)
                        :-d.x1806
                        (:-d.make-syntax-object1372 :-d.x1806 :-d.w1807))))))
              (:-d.unwrap1338
                (lambda (:-d.x1808)
                  (if (:-d.syntax-object?1373 :-d.x1808)
                    ((lambda (:-d.e1809 :-d.w1810)
                       (if (pair? :-d.e1809)
                         (cons (:-d.wrap1337 (car :-d.e1809) :-d.w1810)
                               (:-d.wrap1337 (cdr :-d.e1809) :-d.w1810))
                         (if (vector? :-d.e1809)
                           (list->vector
                             (map (lambda (:-d.x1811)
                                    (:-d.wrap1337 :-d.x1811 :-d.w1810))
                                  (vector->list :-d.e1809)))
                           :-d.e1809)))
                     (:-d.syntax-object-expression1371 :-d.x1808)
                     (:-d.syntax-object-wrap1370 :-d.x1808))
                    :-d.x1808)))
              (:-d.bound-id-member?1339
                (lambda (:-d.x1812 :-d.list1813)
                  (if (not (null? :-d.list1813))
                    ((lambda (:-d.t1814)
                       (if :-d.t1814
                         :-d.t1814
                         (:-d.bound-id-member?1339
                           :-d.x1812
                           (cdr :-d.list1813))))
                     (:-d.bound-id=?1341 :-d.x1812 (car :-d.list1813)))
                    '#f)))
              (:-d.valid-bound-ids?1340
                (lambda (:-d.ids1815)
                  (if ((letrec ((:-d.all-ids?1819
                                  (lambda (:-d.ids1820)
                                    ((lambda (:-d.t1821)
                                       (if :-d.t1821
                                         :-d.t1821
                                         (if (:-d.id?1356 (car :-d.ids1820))
                                           (:-d.all-ids?1819 (cdr :-d.ids1820))
                                           '#f)))
                                     (null? :-d.ids1820)))))
                         :-d.all-ids?1819)
                       :-d.ids1815)
                    ((letrec ((:-d.unique?1816
                                (lambda (:-d.ids1817)
                                  ((lambda (:-d.t1818)
                                     (if :-d.t1818
                                       :-d.t1818
                                       (if (not (:-d.bound-id-member?1339
                                                  (car :-d.ids1817)
                                                  (cdr :-d.ids1817)))
                                         (:-d.unique?1816 (cdr :-d.ids1817))
                                         '#f)))
                                   (null? :-d.ids1817)))))
                       :-d.unique?1816)
                     :-d.ids1815)
                    '#f)))
              (:-d.bound-id=?1341
                (lambda (:-d.i1822 :-d.j1823)
                  (if (eq? (:-d.id-sym-name1355 :-d.i1822)
                           (:-d.id-sym-name1355 :-d.j1823))
                    ((lambda (:-d.i1824 :-d.j1825)
                       (if (eq? (car :-d.i1824) (car :-d.j1825))
                         (:-d.same-marks?1345 (cdr :-d.i1824) (cdr :-d.j1825))
                         '#f))
                     (:-d.id-var-name&marks1343 :-d.i1822 :-d.empty-wrap1354)
                     (:-d.id-var-name&marks1343 :-d.j1823 :-d.empty-wrap1354))
                    '#f)))
              (:-d.free-id=?1342
                (lambda (:-d.i1826 :-d.j1827)
                  (if (eq? (:-d.id-sym-name1355 :-d.i1826)
                           (:-d.id-sym-name1355 :-d.j1827))
                    (eq? (:-d.id-var-name1344 :-d.i1826 :-d.empty-wrap1354)
                         (:-d.id-var-name1344 :-d.j1827 :-d.empty-wrap1354))
                    '#f)))
              (:-d.id-var-name&marks1343
                (lambda (:-d.id1828 :-d.w1829)
                  (if (null? :-d.w1829)
                    (if (symbol? :-d.id1828)
                      (list :-d.id1828)
                      (:-d.id-var-name&marks1343
                        (:-d.syntax-object-expression1371 :-d.id1828)
                        (:-d.syntax-object-wrap1370 :-d.id1828)))
                    ((lambda (:-d.n&m1830 :-d.first1831)
                       (if (pair? :-d.first1831)
                         ((lambda (:-d.n1833)
                            ((letrec ((:-d.search1834
                                        (lambda (:-d.rib1835)
                                          (if (null? :-d.rib1835)
                                            :-d.n&m1830
                                            (if (if (eq? (caar :-d.rib1835)
                                                         :-d.n1833)
                                                  (:-d.same-marks?1345
                                                    (cdr :-d.n&m1830)
                                                    (cddar :-d.rib1835))
                                                  '#f)
                                              (cdar :-d.rib1835)
                                              (:-d.search1834
                                                (cdr :-d.rib1835)))))))
                               :-d.search1834)
                             :-d.first1831))
                          (car :-d.n&m1830))
                         (cons (car :-d.n&m1830)
                               (if ((lambda (:-d.t1832)
                                      (if :-d.t1832
                                        :-d.t1832
                                        (not (eqv? :-d.first1831
                                                   (cadr :-d.n&m1830)))))
                                    (null? (cdr :-d.n&m1830)))
                                 (cons :-d.first1831 (cdr :-d.n&m1830))
                                 (cddr :-d.n&m1830)))))
                     (:-d.id-var-name&marks1343 :-d.id1828 (cdr :-d.w1829))
                     (car :-d.w1829)))))
              (:-d.id-var-name1344
                (lambda (:-d.id1836 :-d.w1837)
                  (if (null? :-d.w1837)
                    (if (symbol? :-d.id1836)
                      :-d.id1836
                      (:-d.id-var-name1344
                        (:-d.syntax-object-expression1371 :-d.id1836)
                        (:-d.syntax-object-wrap1370 :-d.id1836)))
                    (if (pair? (car :-d.w1837))
                      (car (:-d.id-var-name&marks1343 :-d.id1836 :-d.w1837))
                      (:-d.id-var-name1344 :-d.id1836 (cdr :-d.w1837))))))
              (:-d.same-marks?1345
                (lambda (:-d.x1838 :-d.y1839)
                  (if (null? :-d.x1838)
                    (null? :-d.y1839)
                    (if (not (null? :-d.y1839))
                      (if (eqv? (car :-d.x1838) (car :-d.y1839))
                        (:-d.same-marks?1345 (cdr :-d.x1838) (cdr :-d.y1839))
                        '#f)
                      '#f))))
              (:-d.join-wraps21346
                (lambda (:-d.w11840 :-d.w21841)
                  ((lambda (:-d.x1842 :-d.w11843)
                     (if (null? :-d.w11843)
                       (if (if (not (pair? :-d.x1842))
                             (eqv? :-d.x1842 (car :-d.w21841))
                             '#f)
                         (cdr :-d.w21841)
                         (cons :-d.x1842 :-d.w21841))
                       (cons :-d.x1842
                             (:-d.join-wraps21346 :-d.w11843 :-d.w21841))))
                   (car :-d.w11840)
                   (cdr :-d.w11840))))
              (:-d.join-wraps11347
                (lambda (:-d.w11844 :-d.w21845)
                  (if (null? :-d.w11844)
                    :-d.w21845
                    (cons (car :-d.w11844)
                          (:-d.join-wraps11347 (cdr :-d.w11844) :-d.w21845)))))
              (:-d.join-wraps1348
                (lambda (:-d.w11846 :-d.w21847)
                  (if (null? :-d.w21847)
                    :-d.w11846
                    (if (null? :-d.w11846)
                      :-d.w21847
                      (if (pair? (car :-d.w21847))
                        (:-d.join-wraps11347 :-d.w11846 :-d.w21847)
                        (:-d.join-wraps21346 :-d.w11846 :-d.w21847))))))
              (:-d.make-wrap-rib1349
                (lambda (:-d.ids1848 :-d.new-names1849 :-d.w1850)
                  (if (null? :-d.ids1848)
                    '()
                    (cons ((lambda (:-d.n&m1851)
                             (cons (car :-d.n&m1851)
                                   (cons (car :-d.new-names1849)
                                         (cdr :-d.n&m1851))))
                           (:-d.id-var-name&marks1343
                             (car :-d.ids1848)
                             :-d.w1850))
                          (:-d.make-wrap-rib1349
                            (cdr :-d.ids1848)
                            (cdr :-d.new-names1849)
                            :-d.w1850)))))
              (:-d.make-binding-wrap1350
                (lambda (:-d.ids1852 :-d.new-names1853 :-d.w1854)
                  (if (null? :-d.ids1852)
                    :-d.w1854
                    (cons (:-d.make-wrap-rib1349
                            :-d.ids1852
                            :-d.new-names1853
                            :-d.w1854)
                          :-d.w1854))))
              (:-d.new-mark-wrap1351
                (lambda ()
                  (begin
                    (set! :-d.current-mark1352 (+ :-d.current-mark1352 '1))
                    (list :-d.current-mark1352))))
              (:-d.current-mark1352 '0)
              (:-d.top-wrap1353 '(top))
              (:-d.empty-wrap1354 '())
              (:-d.id-sym-name1355
                (lambda (:-d.x1855)
                  (if (symbol? :-d.x1855)
                    :-d.x1855
                    (:-d.syntax-object-expression1371 :-d.x1855))))
              (:-d.id?1356
                (lambda (:-d.x1856)
                  ((lambda (:-d.t1857)
                     (if :-d.t1857
                       :-d.t1857
                       (if (:-d.syntax-object?1373 :-d.x1856)
                         (symbol? (:-d.syntax-object-expression1371 :-d.x1856))
                         '#f)))
                   (symbol? :-d.x1856))))
              (:-d.global-extend1357
                (lambda (:-d.type1858 :-d.sym1859 :-d.val1860)
                  (:-d.extend-global-env1364
                    :-d.sym1859
                    (cons :-d.type1858 :-d.val1860))))
              (:-d.lookup1358
                (lambda (:-d.name1861 :-d.id1862 :-d.r1863)
                  (if (eq? :-d.name1861 (:-d.id-sym-name1355 :-d.id1862))
                    (:-d.global-lookup1363 :-d.name1861)
                    ((letrec ((:-d.search1864
                                (lambda (:-d.r1865 :-d.name1866)
                                  (if (null? :-d.r1865)
                                    '(displaced-lexical)
                                    (if (pair? (car :-d.r1865))
                                      (if (eq? (caar :-d.r1865) :-d.name1866)
                                        (cdar :-d.r1865)
                                        (:-d.search1864
                                          (cdr :-d.r1865)
                                          :-d.name1866))
                                      (if (eq? (car :-d.r1865) :-d.name1866)
                                        '(lexical)
                                        (:-d.search1864
                                          (cdr :-d.r1865)
                                          :-d.name1866)))))))
                       :-d.search1864)
                     :-d.r1863
                     :-d.name1861))))
              (:-d.extend-syntax-env1359
                (lambda (:-d.vars1867 :-d.vals1868 :-d.r1869)
                  (if (null? :-d.vars1867)
                    :-d.r1869
                    (cons (cons (car :-d.vars1867)
                                (cons 'syntax (car :-d.vals1868)))
                          (:-d.extend-syntax-env1359
                            (cdr :-d.vars1867)
                            (cdr :-d.vals1868)
                            :-d.r1869)))))
              (:-d.extend-var-env1360 append)
              (:-d.extend-macro-env1361
                (lambda (:-d.vars1870 :-d.vals1871 :-d.r1872)
                  (if (null? :-d.vars1870)
                    :-d.r1872
                    (cons (cons (car :-d.vars1870)
                                (cons 'macro (car :-d.vals1871)))
                          (:-d.extend-macro-env1361
                            (cdr :-d.vars1870)
                            (cdr :-d.vals1871)
                            :-d.r1872)))))
              (:-d.null-env1362 '())
              (:-d.global-lookup1363
                (lambda (:-d.sym1873)
                  ((lambda (:-d.t1874)
                     (if :-d.t1874 :-d.t1874 '(global-unbound)))
                   (get-global-definition-hook :-d.sym1873))))
              (:-d.extend-global-env1364
                (lambda (:-d.sym1875 :-d.binding1876)
                  (put-global-definition-hook :-d.sym1875 :-d.binding1876)))
              (:-d.binding-value1365 cdr)
              (:-d.binding-type1366 car)
              (:-d.arg-check1367
                (lambda (:-d.pred?1877 :-d.x1878 :-d.who1879)
                  (if (not (:-d.pred?1877 :-d.x1878))
                    (error-hook :-d.who1879 '"invalid argument" :-d.x1878)
                    (void))))
              (:-d.id-error1368
                (lambda (:-d.x1880)
                  (syntax-error :-d.x1880 '"invalid context for identifier")))
              (:-d.scope-error1369
                (lambda (:-d.id1881)
                  (syntax-error
                    :-d.id1881
                    '"invalid context for bound identifier")))
              (:-d.syntax-object-wrap1370
                (lambda (:-d.x1882) (vector-ref :-d.x1882 '2)))
              (:-d.syntax-object-expression1371
                (lambda (:-d.x1883) (vector-ref :-d.x1883 '1)))
              (:-d.make-syntax-object1372
                (lambda (:-d.expression1884 :-d.wrap1885)
                  (vector 'syntax-object :-d.expression1884 :-d.wrap1885)))
              (:-d.syntax-object?1373
                (lambda (:-d.x1886)
                  (if (vector? :-d.x1886)
                    (if (= (vector-length :-d.x1886) '3)
                      (eq? (vector-ref :-d.x1886 '0) 'syntax-object)
                      '#f)
                    '#f))))
       (begin
         (:-d.global-extend1357 'core 'letrec-syntax :-d.chi-local-syntax1328)
         (begin
           (:-d.global-extend1357 'core 'let-syntax :-d.chi-local-syntax1328)
           (begin
             (:-d.global-extend1357
               'core
               'quote
               (lambda (:-d.e1374 :-d.r1375 :-d.w1376)
                 ((lambda (:-d.x1377)
                    ((lambda (:-d.y1378)
                       (if :-d.y1378
                         ((lambda (:-d._1382 :-d.e1383)
                            (build-data (:-d.strip1316 :-d.e1383)))
                          (car :-d.y1378)
                          (cadr :-d.y1378))
                         ((lambda (:-d.x1379)
                            ((lambda (:-d.y1380)
                               (if :-d.y1380
                                 ((lambda (:-d._1381)
                                    (syntax-error
                                      (:-d.wrap1337 :-d.e1374 :-d.w1376)))
                                  (car :-d.y1380))
                                 (syntax-error :-d.x1379)))
                             (syntax-dispatch :-d.x1379 '(any) '#())))
                          :-d.x1377)))
                     (syntax-dispatch
                       :-d.x1377
                       '(pair (any) pair (any) atom)
                       '#())))
                  :-d.e1374)))
             (begin
               (:-d.global-extend1357
                 'core
                 'syntax
                 (lambda (:-d.e1384 :-d.r1385 :-d.w1386)
                   ((lambda (:-d.x1387)
                      ((lambda (:-d.y1388)
                         (if :-d.y1388
                           ((lambda (:-d._1392 :-d.x1393)
                              (:-d.chi-syntax1323
                                :-d.x1393
                                :-d.r1385
                                :-d.w1386))
                            (car :-d.y1388)
                            (cadr :-d.y1388))
                           ((lambda (:-d.x1389)
                              ((lambda (:-d.y1390)
                                 (if :-d.y1390
                                   ((lambda (:-d._1391)
                                      (syntax-error
                                        (:-d.wrap1337 :-d.e1384 :-d.w1386)))
                                    (car :-d.y1390))
                                   (syntax-error :-d.x1389)))
                               (syntax-dispatch :-d.x1389 '(any) '#())))
                            :-d.x1387)))
                       (syntax-dispatch
                         :-d.x1387
                         '(pair (any) pair (any) atom)
                         '#())))
                    :-d.e1384)))
               (begin
                 (:-d.global-extend1357
                   'core
                   'syntax-lambda
                   (lambda (:-d.e1394 :-d.r1395 :-d.w1396)
                     ((lambda (:-d.x1397)
                        ((lambda (:-d.dorest1401)
                           ((lambda (:-d.y1402)
                              (if :-d.y1402
                                ((lambda (:-d._1403
                                          :-d.id1404
                                          :-d.level1405
                                          :-d.exp1406)
                                   (if (if (:-d.valid-bound-ids?1340
                                             :-d.id1404)
                                         (map (lambda (:-d.x1412)
                                                (if (integer? :-d.x1412)
                                                  (if (exact? :-d.x1412)
                                                    (not (negative? :-d.x1412))
                                                    '#f)
                                                  '#f))
                                              (map :-d.unwrap1338
                                                   :-d.level1405))
                                         '#f)
                                     ((lambda (:-d.new-vars1408)
                                        (build-lambda
                                          :-d.new-vars1408
                                          (:-d.chi1335
                                            :-d.exp1406
                                            (:-d.extend-syntax-env1359
                                              :-d.new-vars1408
                                              (map :-d.unwrap1338
                                                   :-d.level1405)
                                              :-d.r1395)
                                            (:-d.make-binding-wrap1350
                                              :-d.id1404
                                              :-d.new-vars1408
                                              :-d.w1396))))
                                      (map :-d.gen-var1314 :-d.id1404))
                                     (:-d.dorest1401)))
                                 (car :-d.y1402)
                                 (cadr :-d.y1402)
                                 (caddr :-d.y1402)
                                 (cadddr :-d.y1402))
                                (:-d.dorest1401)))
                            (syntax-dispatch
                              :-d.x1397
                              '(pair (any)
                                     pair
                                     (each pair (any) pair (any) atom)
                                     pair
                                     (any)
                                     atom)
                              '#())))
                         (lambda ()
                           ((lambda (:-d.x1398)
                              ((lambda (:-d.y1399)
                                 (if :-d.y1399
                                   ((lambda (:-d._1400)
                                      (syntax-error
                                        (:-d.wrap1337 :-d.e1394 :-d.w1396)))
                                    (car :-d.y1399))
                                   (syntax-error :-d.x1398)))
                               (syntax-dispatch :-d.x1398 '(any) '#())))
                            :-d.x1397))))
                      :-d.e1394)))
                 (begin
                   (:-d.global-extend1357
                     'core
                     'lambda
                     (lambda (:-d.e1414 :-d.r1415 :-d.w1416)
                       ((lambda (:-d.x1417)
                          ((lambda (:-d.y1418)
                             (if :-d.y1418
                               ((lambda (:-d._1431
                                         :-d.id1432
                                         :-d.e11433
                                         :-d.e21434)
                                  (if (not (:-d.valid-bound-ids?1340
                                             :-d.id1432))
                                    (syntax-error
                                      (:-d.wrap1337 :-d.e1414 :-d.w1416)
                                      '"invalid parameter list")
                                    ((lambda (:-d.new-vars1436)
                                       (build-lambda
                                         :-d.new-vars1436
                                         (:-d.chi-body1329
                                           (cons :-d.e11433 :-d.e21434)
                                           :-d.e1414
                                           (:-d.extend-var-env1360
                                             :-d.new-vars1436
                                             :-d.r1415)
                                           (:-d.make-binding-wrap1350
                                             :-d.id1432
                                             :-d.new-vars1436
                                             :-d.w1416))))
                                     (map :-d.gen-var1314 :-d.id1432))))
                                (car :-d.y1418)
                                (cadr :-d.y1418)
                                (caddr :-d.y1418)
                                (cadddr :-d.y1418))
                               ((lambda (:-d.x1419)
                                  ((lambda (:-d.y1420)
                                     (if :-d.y1420
                                       ((lambda (:-d._1424
                                                 :-d.ids1425
                                                 :-d.e11426
                                                 :-d.e21427)
                                          ((lambda (:-d.old-ids1428)
                                             (if (not (:-d.valid-bound-ids?1340
                                                        (:-d.lambda-var-list1313
                                                          :-d.ids1425)))
                                               (syntax-error
                                                 (:-d.wrap1337
                                                   :-d.e1414
                                                   :-d.w1416)
                                                 '"invalid parameter list")
                                               ((lambda (:-d.new-vars1429)
                                                  (build-improper-lambda
                                                    (reverse (cdr :-d.new-vars1429))
                                                    (car :-d.new-vars1429)
                                                    (:-d.chi-body1329
                                                      (cons :-d.e11426
                                                            :-d.e21427)
                                                      :-d.e1414
                                                      (:-d.extend-var-env1360
                                                        :-d.new-vars1429
                                                        :-d.r1415)
                                                      (:-d.make-binding-wrap1350
                                                        :-d.old-ids1428
                                                        :-d.new-vars1429
                                                        :-d.w1416))))
                                                (map :-d.gen-var1314
                                                     :-d.old-ids1428))))
                                           (:-d.lambda-var-list1313
                                             :-d.ids1425)))
                                        (car :-d.y1420)
                                        (cadr :-d.y1420)
                                        (caddr :-d.y1420)
                                        (cadddr :-d.y1420))
                                       ((lambda (:-d.x1421)
                                          ((lambda (:-d.y1422)
                                             (if :-d.y1422
                                               ((lambda (:-d._1423)
                                                  (syntax-error
                                                    (:-d.wrap1337
                                                      :-d.e1414
                                                      :-d.w1416)))
                                                (car :-d.y1422))
                                               (syntax-error :-d.x1421)))
                                           (syntax-dispatch
                                             :-d.x1421
                                             '(any)
                                             '#())))
                                        :-d.x1419)))
                                   (syntax-dispatch
                                     :-d.x1419
                                     '(pair (any)
                                            pair
                                            (any)
                                            pair
                                            (any)
                                            each
                                            any)
                                     '#())))
                                :-d.x1417)))
                           (syntax-dispatch
                             :-d.x1417
                             '(pair (any) pair (each any) pair (any) each any)
                             '#())))
                        :-d.e1414)))
                   (begin
                     (:-d.global-extend1357
                       'core
                       'letrec
                       (lambda (:-d.e1440 :-d.r1441 :-d.w1442)
                         ((lambda (:-d.x1443)
                            ((lambda (:-d.dorest1447)
                               ((lambda (:-d.y1448)
                                  (if :-d.y1448
                                    (apply (lambda (:-d._1449
                                                    :-d.id1450
                                                    :-d.val1451
                                                    :-d.e11452
                                                    :-d.e21453)
                                             (if (:-d.valid-bound-ids?1340
                                                   :-d.id1450)
                                               ((lambda (:-d.new-vars1455)
                                                  ((lambda (:-d.w1457
                                                            :-d.r1458)
                                                     (build-letrec
                                                       :-d.new-vars1455
                                                       (map (lambda (:-d.x1461)
                                                              (:-d.chi1335
                                                                :-d.x1461
                                                                :-d.r1458
                                                                :-d.w1457))
                                                            :-d.val1451)
                                                       (:-d.chi-body1329
                                                         (cons :-d.e11452
                                                               :-d.e21453)
                                                         :-d.e1440
                                                         :-d.r1458
                                                         :-d.w1457)))
                                                   (:-d.make-binding-wrap1350
                                                     :-d.id1450
                                                     :-d.new-vars1455
                                                     :-d.w1442)
                                                   (:-d.extend-var-env1360
                                                     :-d.new-vars1455
                                                     :-d.r1441)))
                                                (map :-d.gen-var1314
                                                     :-d.id1450))
                                               (:-d.dorest1447)))
                                           :-d.y1448)
                                    (:-d.dorest1447)))
                                (syntax-dispatch
                                  :-d.x1443
                                  '(pair (any)
                                         pair
                                         (each pair (any) pair (any) atom)
                                         pair
                                         (any)
                                         each
                                         any)
                                  '#())))
                             (lambda ()
                               ((lambda (:-d.x1444)
                                  ((lambda (:-d.y1445)
                                     (if :-d.y1445
                                       ((lambda (:-d._1446)
                                          (syntax-error
                                            (:-d.wrap1337
                                              :-d.e1440
                                              :-d.w1442)))
                                        (car :-d.y1445))
                                       (syntax-error :-d.x1444)))
                                   (syntax-dispatch :-d.x1444 '(any) '#())))
                                :-d.x1443))))
                          :-d.e1440)))
                     (begin
                       (:-d.global-extend1357
                         'core
                         'if
                         (lambda (:-d.e1463 :-d.r1464 :-d.w1465)
                           ((lambda (:-d.x1466)
                              ((lambda (:-d.y1467)
                                 (if :-d.y1467
                                   ((lambda (:-d._1477
                                             :-d.test1478
                                             :-d.then1479)
                                      (build-conditional
                                        (:-d.chi1335
                                          :-d.test1478
                                          :-d.r1464
                                          :-d.w1465)
                                        (:-d.chi1335
                                          :-d.then1479
                                          :-d.r1464
                                          :-d.w1465)
                                        (:-d.chi1335
                                          (list 'void)
                                          :-d.r1464
                                          :-d.empty-wrap1354)))
                                    (car :-d.y1467)
                                    (cadr :-d.y1467)
                                    (caddr :-d.y1467))
                                   ((lambda (:-d.x1468)
                                      ((lambda (:-d.y1469)
                                         (if :-d.y1469
                                           ((lambda (:-d._1473
                                                     :-d.test1474
                                                     :-d.then1475
                                                     :-d.else1476)
                                              (build-conditional
                                                (:-d.chi1335
                                                  :-d.test1474
                                                  :-d.r1464
                                                  :-d.w1465)
                                                (:-d.chi1335
                                                  :-d.then1475
                                                  :-d.r1464
                                                  :-d.w1465)
                                                (:-d.chi1335
                                                  :-d.else1476
                                                  :-d.r1464
                                                  :-d.w1465)))
                                            (car :-d.y1469)
                                            (cadr :-d.y1469)
                                            (caddr :-d.y1469)
                                            (cadddr :-d.y1469))
                                           ((lambda (:-d.x1470)
                                              ((lambda (:-d.y1471)
                                                 (if :-d.y1471
                                                   ((lambda (:-d._1472)
                                                      (syntax-error
                                                        (:-d.wrap1337
                                                          :-d.e1463
                                                          :-d.w1465)))
                                                    (car :-d.y1471))
                                                   (syntax-error :-d.x1470)))
                                               (syntax-dispatch
                                                 :-d.x1470
                                                 '(any)
                                                 '#())))
                                            :-d.x1468)))
                                       (syntax-dispatch
                                         :-d.x1468
                                         '(pair (any)
                                                pair
                                                (any)
                                                pair
                                                (any)
                                                pair
                                                (any)
                                                atom)
                                         '#())))
                                    :-d.x1466)))
                               (syntax-dispatch
                                 :-d.x1466
                                 '(pair (any) pair (any) pair (any) atom)
                                 '#())))
                            :-d.e1463)))
                       (begin
                         (:-d.global-extend1357
                           'core
                           'set!
                           (lambda (:-d.e1480 :-d.r1481 :-d.w1482)
                             ((lambda (:-d.x1483)
                                ((lambda (:-d.dorest1487)
                                   ((lambda (:-d.y1488)
                                      (if :-d.y1488
                                        ((lambda (:-d._1489
                                                  :-d.id1490
                                                  :-d.val1491)
                                           (if (:-d.id?1356 :-d.id1490)
                                             ((lambda (:-d.val1492 :-d.n1493)
                                                ((lambda (:-d.x1494)
                                                   (if (memv :-d.x1494
                                                             '(lexical))
                                                     (build-lexical-assignment
                                                       :-d.n1493
                                                       :-d.val1492)
                                                     ((lambda (:-d.x1495)
                                                        (if (memv :-d.x1495
                                                                  '(global global-unbound))
                                                          (build-global-assignment
                                                            :-d.n1493
                                                            :-d.val1492)
                                                          (begin
                                                            :-d.x1495
                                                            (:-d.id-error1368
                                                              (:-d.wrap1337
                                                                :-d.id1490
                                                                :-d.w1482)))))
                                                      :-d.x1494)))
                                                 (:-d.binding-type1366
                                                   (:-d.lookup1358
                                                     :-d.n1493
                                                     :-d.id1490
                                                     :-d.r1481))))
                                              (:-d.chi1335
                                                :-d.val1491
                                                :-d.r1481
                                                :-d.w1482)
                                              (:-d.id-var-name1344
                                                :-d.id1490
                                                :-d.w1482))
                                             (:-d.dorest1487)))
                                         (car :-d.y1488)
                                         (cadr :-d.y1488)
                                         (caddr :-d.y1488))
                                        (:-d.dorest1487)))
                                    (syntax-dispatch
                                      :-d.x1483
                                      '(pair (any) pair (any) pair (any) atom)
                                      '#())))
                                 (lambda ()
                                   ((lambda (:-d.x1484)
                                      ((lambda (:-d.y1485)
                                         (if :-d.y1485
                                           ((lambda (:-d._1486)
                                              (syntax-error
                                                (:-d.wrap1337
                                                  :-d.e1480
                                                  :-d.w1482)))
                                            (car :-d.y1485))
                                           (syntax-error :-d.x1484)))
                                       (syntax-dispatch
                                         :-d.x1484
                                         '(any)
                                         '#())))
                                    :-d.x1483))))
                              :-d.e1480)))
                         (begin
                           (:-d.global-extend1357
                             'special
                             'begin
                             (lambda (:-d.e1496 :-d.r1497 :-d.w1498 :-d.k1499)
                               ((lambda (:-d.body1500)
                                  (if (null? :-d.body1500)
                                    (if (eqv? :-d.k1499 :-d.chi-top1336)
                                      (:-d.chi1335
                                        (list 'void)
                                        :-d.r1497
                                        :-d.empty-wrap1354)
                                      (syntax-error
                                        (:-d.wrap1337 :-d.e1496 :-d.w1498)
                                        '"no expressions in body of"))
                                    (build-sequence
                                      ((letrec ((:-d.dobody1501
                                                  (lambda (:-d.body1502)
                                                    (if (null? :-d.body1502)
                                                      '()
                                                      ((lambda (:-d.first1503)
                                                         (cons :-d.first1503
                                                               (:-d.dobody1501
                                                                 (cdr :-d.body1502))))
                                                       (:-d.k1499
                                                         (car :-d.body1502)
                                                         :-d.r1497
                                                         :-d.empty-wrap1354))))))
                                         :-d.dobody1501)
                                       :-d.body1500))))
                                (:-d.chi-sequence1326 :-d.e1496 :-d.w1498))))
                           (begin
                             (:-d.global-extend1357
                               'special
                               'define
                               (lambda (:-d.e1504
                                        :-d.r1505
                                        :-d.w1506
                                        :-d.k1507)
                                 (if (eqv? :-d.k1507 :-d.chi-top1336)
                                   ((lambda (:-d.n&v1508)
                                      ((lambda (:-d.n1509)
                                         (begin
                                           (:-d.global-extend1357
                                             'global
                                             :-d.n1509
                                             '())
                                           (build-global-definition
                                             :-d.n1509
                                             (:-d.chi1335
                                               (cadr :-d.n&v1508)
                                               :-d.r1505
                                               :-d.empty-wrap1354))))
                                       (:-d.id-var-name1344
                                         (car :-d.n&v1508)
                                         :-d.empty-wrap1354)))
                                    (:-d.chi-definition1325
                                      :-d.e1504
                                      :-d.w1506))
                                   (syntax-error
                                     (:-d.wrap1337 :-d.e1504 :-d.w1506)
                                     '"invalid context for definition"))))
                             (begin
                               (:-d.global-extend1357
                                 'special
                                 'define-syntax
                                 (lambda (:-d.e1510
                                          :-d.r1511
                                          :-d.w1512
                                          :-d.k1513)
                                   (if (eqv? :-d.k1513 :-d.chi-top1336)
                                     ((lambda (:-d.n&v1514)
                                        (begin
                                          (:-d.global-extend1357
                                            'macro
                                            (:-d.id-var-name1344
                                              (car :-d.n&v1514)
                                              :-d.empty-wrap1354)
                                            (:-d.chi-macro-def1327
                                              (cadr :-d.n&v1514)
                                              :-d.r1511
                                              :-d.empty-wrap1354))
                                          (:-d.chi1335
                                            (list 'void)
                                            :-d.r1511
                                            :-d.empty-wrap1354)))
                                      (:-d.chi-syntax-definition1324
                                        :-d.e1510
                                        :-d.w1512))
                                     (syntax-error
                                       (:-d.wrap1337 :-d.e1510 :-d.w1512)
                                       '"invalid context for definition"))))
                               (begin
                                 (set! expand-syntax
                                   (lambda (:-d.x1515)
                                     (:-d.chi-top1336
                                       :-d.x1515
                                       :-d.null-env1362
                                       :-d.top-wrap1353)))
                                 (begin
                                   (set! implicit-identifier
                                     (lambda (:-d.id1516 :-d.sym1517)
                                       (begin
                                         (:-d.arg-check1367
                                           :-d.id?1356
                                           :-d.id1516
                                           'implicit-identifier)
                                         (begin
                                           (:-d.arg-check1367
                                             symbol?
                                             :-d.sym1517
                                             'implicit-identifier)
                                           (if (:-d.syntax-object?1373
                                                 :-d.id1516)
                                             (:-d.wrap1337
                                               :-d.sym1517
                                               (:-d.syntax-object-wrap1370
                                                 :-d.id1516))
                                             :-d.sym1517)))))
                                   (begin
                                     (set! syntax-object->datum
                                       (lambda (:-d.x1518)
                                         (:-d.strip1316 :-d.x1518)))
                                     (begin
                                       (set! generate-temporaries
                                         (lambda (:-d.ls1519)
                                           (begin
                                             (:-d.arg-check1367
                                               list?
                                               :-d.ls1519
                                               'generate-temporaries)
                                             (map (lambda (:-d.x1520) (gensym))
                                                  :-d.ls1519))))
                                       (begin
                                         (set! free-identifier=?
                                           (lambda (:-d.x1521 :-d.y1522)
                                             (begin
                                               (:-d.arg-check1367
                                                 :-d.id?1356
                                                 :-d.x1521
                                                 'free-identifier=?)
                                               (begin
                                                 (:-d.arg-check1367
                                                   :-d.id?1356
                                                   :-d.y1522
                                                   'free-identifier=?)
                                                 (:-d.free-id=?1342
                                                   :-d.x1521
                                                   :-d.y1522)))))
                                         (begin
                                           (set! bound-identifier=?
                                             (lambda (:-d.x1523 :-d.y1524)
                                               (begin
                                                 (:-d.arg-check1367
                                                   :-d.id?1356
                                                   :-d.x1523
                                                   'bound-identifier=?)
                                                 (begin
                                                   (:-d.arg-check1367
                                                     :-d.id?1356
                                                     :-d.y1524
                                                     'bound-identifier=?)
                                                   (:-d.bound-id=?1341
                                                     :-d.x1523
                                                     :-d.y1524)))))
                                           (begin
                                             (set! identifier?
                                               (lambda (:-d.x1525)
                                                 (:-d.id?1356 :-d.x1525)))
                                             (begin
                                               (set! syntax-error
                                                 (lambda (:-d.object1527
                                                          .
                                                          :-d.messages1526)
                                                   (begin
                                                     (for-each
                                                       (lambda (:-d.x1528)
                                                         (:-d.arg-check1367
                                                           string?
                                                           :-d.x1528
                                                           'syntax-error))
                                                       :-d.messages1526)
                                                     ((lambda (:-d.message1529)
                                                        (error-hook
                                                          'expand-syntax
                                                          :-d.message1529
                                                          (:-d.strip1316
                                                            :-d.object1527)))
                                                      (if (null? :-d.messages1526)
                                                        '"invalid syntax"
                                                        (apply string-append
                                                               :-d.messages1526))))))
                                               (begin
                                                 (set! install-global-transformer
                                                   (lambda (:-d.sym1530
                                                            :-d.p1531)
                                                     (:-d.global-extend1357
                                                       'macro
                                                       :-d.sym1530
                                                       :-d.p1531)))
                                                 ((lambda ()
                                                    (letrec ((:-d.match1532
                                                               (lambda (:-d.e1540
                                                                        :-d.p1541
                                                                        :-d.k1542
                                                                        :-d.w1543
                                                                        :-d.r1544)
                                                                 (if :-d.r1544
                                                                   ((lambda (:-d.x1545)
                                                                      (if (memv :-d.x1545
                                                                                '(any))
                                                                        (cons (:-d.wrap1337
                                                                                :-d.e1540
                                                                                :-d.w1543)
                                                                              :-d.r1544)
                                                                        ((lambda (:-d.x1546)
                                                                           (if (memv :-d.x1546
                                                                                     '(free-id))
                                                                             (if (identifier?
                                                                                   :-d.e1540)
                                                                               (if (free-identifier=?
                                                                                     (:-d.wrap1337
                                                                                       :-d.e1540
                                                                                       :-d.w1543)
                                                                                     (vector-ref
                                                                                       :-d.k1542
                                                                                       (cdr :-d.p1541)))
                                                                                 :-d.r1544
                                                                                 '#f)
                                                                               '#f)
                                                                             (begin
                                                                               :-d.x1546
                                                                               (if (:-d.syntax-object?1373
                                                                                     :-d.e1540)
                                                                                 (:-d.match*1533
                                                                                   (:-d.syntax-object-expression1371
                                                                                     :-d.e1540)
                                                                                   :-d.p1541
                                                                                   :-d.k1542
                                                                                   (:-d.join-wraps1348
                                                                                     :-d.w1543
                                                                                     (:-d.syntax-object-wrap1370
                                                                                       :-d.e1540))
                                                                                   :-d.r1544)
                                                                                 (:-d.match*1533
                                                                                   :-d.e1540
                                                                                   :-d.p1541
                                                                                   :-d.k1542
                                                                                   :-d.w1543
                                                                                   :-d.r1544)))))
                                                                         :-d.x1545)))
                                                                    (car :-d.p1541))
                                                                   '#f)))
                                                             (:-d.match*1533
                                                               (lambda (:-d.e1547
                                                                        :-d.p1548
                                                                        :-d.k1549
                                                                        :-d.w1550
                                                                        :-d.r1551)
                                                                 ((lambda (:-d.x1552)
                                                                    (if (memv :-d.x1552
                                                                              '(pair))
                                                                      (if (pair? :-d.e1547)
                                                                        (:-d.match1532
                                                                          (car :-d.e1547)
                                                                          (cadr :-d.p1548)
                                                                          :-d.k1549
                                                                          :-d.w1550
                                                                          (:-d.match1532
                                                                            (cdr :-d.e1547)
                                                                            (cddr :-d.p1548)
                                                                            :-d.k1549
                                                                            :-d.w1550
                                                                            :-d.r1551))
                                                                        '#f)
                                                                      ((lambda (:-d.x1553)
                                                                         (if (memv :-d.x1553
                                                                                   '(each))
                                                                           (if (eq? (cadr :-d.p1548)
                                                                                    'any)
                                                                             ((lambda (:-d.l1558)
                                                                                (if :-d.l1558
                                                                                  (cons :-d.l1558
                                                                                        :-d.r1551)
                                                                                  '#f))
                                                                              (:-d.match-each-any1535
                                                                                :-d.e1547
                                                                                :-d.w1550))
                                                                             (if (null? :-d.e1547)
                                                                               (:-d.match-empty1534
                                                                                 (cdr :-d.p1548)
                                                                                 :-d.r1551)
                                                                               ((lambda (:-d.l1555)
                                                                                  (if :-d.l1555
                                                                                    ((letrec ((:-d.collect1556
                                                                                                (lambda (:-d.l1557)
                                                                                                  (if (null? (car :-d.l1557))
                                                                                                    :-d.r1551
                                                                                                    (cons (map car
                                                                                                               :-d.l1557)
                                                                                                          (:-d.collect1556
                                                                                                            (map cdr
                                                                                                                 :-d.l1557)))))))
                                                                                       :-d.collect1556)
                                                                                     :-d.l1555)
                                                                                    '#f))
                                                                                (:-d.match-each1536
                                                                                  :-d.e1547
                                                                                  (cdr :-d.p1548)
                                                                                  :-d.k1549
                                                                                  :-d.w1550))))
                                                                           ((lambda (:-d.x1554)
                                                                              (if (memv :-d.x1554
                                                                                        '(atom))
                                                                                (if (equal? (cdr :-d.p1548)
                                                                                            :-d.e1547)
                                                                                  :-d.r1551
                                                                                  '#f)
                                                                                (if (memv :-d.x1554
                                                                                          '(vector))
                                                                                  (if (vector? :-d.e1547)
                                                                                    (:-d.match1532
                                                                                      (vector->list
                                                                                        :-d.e1547)
                                                                                      (cdr :-d.p1548)
                                                                                      :-d.k1549
                                                                                      :-d.w1550
                                                                                      :-d.r1551)
                                                                                    '#f)
                                                                                  (void))))
                                                                            :-d.x1553)))
                                                                       :-d.x1552)))
                                                                  (car :-d.p1548))))
                                                             (:-d.match-empty1534
                                                               (lambda (:-d.p1559
                                                                        :-d.r1560)
                                                                 ((lambda (:-d.x1561)
                                                                    (if (memv :-d.x1561
                                                                              '(any))
                                                                      (cons '()
                                                                            :-d.r1560)
                                                                      ((lambda (:-d.x1562)
                                                                         (if (memv :-d.x1562
                                                                                   '(each))
                                                                           (:-d.match-empty1534
                                                                             (cdr :-d.p1559)
                                                                             :-d.r1560)
                                                                           ((lambda (:-d.x1563)
                                                                              (if (memv :-d.x1563
                                                                                        '(pair))
                                                                                (:-d.match-empty1534
                                                                                  (cadr :-d.p1559)
                                                                                  (:-d.match-empty1534
                                                                                    (cddr :-d.p1559)
                                                                                    :-d.r1560))
                                                                                ((lambda (:-d.x1564)
                                                                                   (if (memv :-d.x1564
                                                                                             '(free-id atom))
                                                                                     :-d.r1560
                                                                                     (if (memv :-d.x1564
                                                                                               '(vector))
                                                                                       (:-d.match-empty1534
                                                                                         (cdr :-d.p1559)
                                                                                         :-d.r1560)
                                                                                       (void))))
                                                                                 :-d.x1563)))
                                                                            :-d.x1562)))
                                                                       :-d.x1561)))
                                                                  (car :-d.p1559))))
                                                             (:-d.match-each-any1535
                                                               (lambda (:-d.e1565
                                                                        :-d.w1566)
                                                                 (if (pair? :-d.e1565)
                                                                   ((lambda (:-d.l1567)
                                                                      (if :-d.l1567
                                                                        (cons (:-d.wrap1337
                                                                                (car :-d.e1565)
                                                                                :-d.w1566)
                                                                              :-d.l1567)
                                                                        '#f))
                                                                    (:-d.match-each-any1535
                                                                      (cdr :-d.e1565)
                                                                      :-d.w1566))
                                                                   (if (null? :-d.e1565)
                                                                     '()
                                                                     (if (:-d.syntax-object?1373
                                                                           :-d.e1565)
                                                                       (:-d.match-each-any1535
                                                                         (:-d.syntax-object-expression1371
                                                                           :-d.e1565)
                                                                         (:-d.join-wraps1348
                                                                           :-d.w1566
                                                                           (:-d.syntax-object-wrap1370
                                                                             :-d.e1565)))
                                                                       '#f)))))
                                                             (:-d.match-each1536
                                                               (lambda (:-d.e1568
                                                                        :-d.p1569
                                                                        :-d.k1570
                                                                        :-d.w1571)
                                                                 (if (pair? :-d.e1568)
                                                                   ((lambda (:-d.first1572)
                                                                      (if :-d.first1572
                                                                        ((lambda (:-d.rest1573)
                                                                           (if :-d.rest1573
                                                                             (cons :-d.first1572
                                                                                   :-d.rest1573)
                                                                             '#f))
                                                                         (:-d.match-each1536
                                                                           (cdr :-d.e1568)
                                                                           :-d.p1569
                                                                           :-d.k1570
                                                                           :-d.w1571))
                                                                        '#f))
                                                                    (:-d.match1532
                                                                      (car :-d.e1568)
                                                                      :-d.p1569
                                                                      :-d.k1570
                                                                      :-d.w1571
                                                                      '()))
                                                                   (if (null? :-d.e1568)
                                                                     '()
                                                                     (if (:-d.syntax-object?1373
                                                                           :-d.e1568)
                                                                       (:-d.match-each1536
                                                                         (:-d.syntax-object-expression1371
                                                                           :-d.e1568)
                                                                         :-d.p1569
                                                                         :-d.k1570
                                                                         (:-d.join-wraps1348
                                                                           :-d.w1571
                                                                           (:-d.syntax-object-wrap1370
                                                                             :-d.e1568)))
                                                                       '#f))))))
                                                      (set! syntax-dispatch
                                                        (lambda (:-d.expression1537
                                                                 :-d.pattern1538
                                                                 :-d.keys1539)
                                                          (:-d.match1532
                                                            :-d.expression1537
                                                            :-d.pattern1538
                                                            :-d.keys1539
                                                            :-d.empty-wrap1354
                                                            '()))))))))))))))))))))))))))))))
  (begin
    (install-global-transformer
      'let
      (lambda (:-d.x1887)
        ((lambda (:-d.x1888)
           ((lambda (:-d.dorest1902)
              ((lambda (:-d.y1903)
                 (if :-d.y1903
                   (apply (lambda (:-d._1904
                                   :-d.x1905
                                   :-d.v1906
                                   :-d.e11907
                                   :-d.e21908)
                            (if (andmap identifier? :-d.x1905)
                              (cons (cons 'lambda
                                          (cons :-d.x1905
                                                (cons :-d.e11907 :-d.e21908)))
                                    :-d.v1906)
                              (:-d.dorest1902)))
                          :-d.y1903)
                   (:-d.dorest1902)))
               (syntax-dispatch
                 :-d.x1888
                 '(pair (any)
                        pair
                        (each pair (any) pair (any) atom)
                        pair
                        (any)
                        each
                        any)
                 '#())))
            (lambda ()
              ((lambda (:-d.x1889)
                 ((lambda (:-d.dorest1890)
                    ((lambda (:-d.y1891)
                       (if :-d.y1891
                         (apply (lambda (:-d._1892
                                         :-d.f1893
                                         :-d.x1894
                                         :-d.v1895
                                         :-d.e11896
                                         :-d.e21897)
                                  (if (andmap identifier?
                                              (cons :-d.f1893 :-d.x1894))
                                    (cons (list 'letrec
                                                (list (list :-d.f1893
                                                            (cons 'lambda
                                                                  (cons :-d.x1894
                                                                        (cons :-d.e11896
                                                                              :-d.e21897)))))
                                                :-d.f1893)
                                          :-d.v1895)
                                    (:-d.dorest1890)))
                                :-d.y1891)
                         (:-d.dorest1890)))
                     (syntax-dispatch
                       :-d.x1889
                       '(pair (any)
                              pair
                              (any)
                              pair
                              (each pair (any) pair (any) atom)
                              pair
                              (any)
                              each
                              any)
                       '#())))
                  (lambda () (syntax-error :-d.x1889))))
               :-d.x1888))))
         :-d.x1887)))
    (install-global-transformer
      'syntax-case
      ((lambda ()
         (letrec ((:-d.build-dispatch-call1913
                    (lambda (:-d.args1983 :-d.body1984 :-d.val1985)
                      ((lambda (:-d.x1986)
                         ((lambda (:-d.y1987)
                            (if :-d.y1987
                              :-d.body1984
                              ((lambda (:-d.x1988)
                                 ((lambda (:-d.y1989)
                                    (if :-d.y1989
                                      ((lambda (:-d.arg12025)
                                         ((lambda (:-d.x2026)
                                            ((lambda (:-d.y2027)
                                               (if :-d.y2027
                                                 ((lambda (:-d.body2028
                                                           :-d.val2029)
                                                    (list (list 'syntax-lambda
                                                                (list :-d.arg12025)
                                                                :-d.body2028)
                                                          (list 'car
                                                                :-d.val2029)))
                                                  (car :-d.y2027)
                                                  (cadr :-d.y2027))
                                                 (syntax-error :-d.x2026)))
                                             (syntax-dispatch
                                               :-d.x2026
                                               '(pair (any) pair (any) atom)
                                               '#())))
                                          (list :-d.body1984 :-d.val1985)))
                                       (car :-d.y1989))
                                      ((lambda (:-d.x1990)
                                         ((lambda (:-d.y1991)
                                            (if :-d.y1991
                                              ((lambda (:-d.arg12019
                                                        :-d.arg22020)
                                                 ((lambda (:-d.x2021)
                                                    ((lambda (:-d.y2022)
                                                       (if :-d.y2022
                                                         ((lambda (:-d.body2023
                                                                   :-d.val2024)
                                                            (list (list 'syntax-lambda
                                                                        (list :-d.arg12019
                                                                              :-d.arg22020)
                                                                        :-d.body2023)
                                                                  (list 'car
                                                                        :-d.val2024)
                                                                  (list 'cadr
                                                                        :-d.val2024)))
                                                          (car :-d.y2022)
                                                          (cadr :-d.y2022))
                                                         (syntax-error
                                                           :-d.x2021)))
                                                     (syntax-dispatch
                                                       :-d.x2021
                                                       '(pair (any)
                                                              pair
                                                              (any)
                                                              atom)
                                                       '#())))
                                                  (list :-d.body1984
                                                        :-d.val1985)))
                                               (car :-d.y1991)
                                               (cadr :-d.y1991))
                                              ((lambda (:-d.x1992)
                                                 ((lambda (:-d.y1993)
                                                    (if :-d.y1993
                                                      ((lambda (:-d.arg12012
                                                                :-d.arg22013
                                                                :-d.arg32014)
                                                         ((lambda (:-d.x2015)
                                                            ((lambda (:-d.y2016)
                                                               (if :-d.y2016
                                                                 ((lambda (:-d.body2017
                                                                           :-d.val2018)
                                                                    (list (list 'syntax-lambda
                                                                                (list :-d.arg12012
                                                                                      :-d.arg22013
                                                                                      :-d.arg32014)
                                                                                :-d.body2017)
                                                                          (list 'car
                                                                                :-d.val2018)
                                                                          (list 'cadr
                                                                                :-d.val2018)
                                                                          (list 'caddr
                                                                                :-d.val2018)))
                                                                  (car :-d.y2016)
                                                                  (cadr :-d.y2016))
                                                                 (syntax-error
                                                                   :-d.x2015)))
                                                             (syntax-dispatch
                                                               :-d.x2015
                                                               '(pair (any)
                                                                      pair
                                                                      (any)
                                                                      atom)
                                                               '#())))
                                                          (list :-d.body1984
                                                                :-d.val1985)))
                                                       (car :-d.y1993)
                                                       (cadr :-d.y1993)
                                                       (caddr :-d.y1993))
                                                      ((lambda (:-d.x1994)
                                                         ((lambda (:-d.y1995)
                                                            (if :-d.y1995
                                                              ((lambda (:-d.arg12004
                                                                        :-d.arg22005
                                                                        :-d.arg32006
                                                                        :-d.arg42007)
                                                                 ((lambda (:-d.x2008)
                                                                    ((lambda (:-d.y2009)
                                                                       (if :-d.y2009
                                                                         ((lambda (:-d.body2010
                                                                                   :-d.val2011)
                                                                            (list (list 'syntax-lambda
                                                                                        (list :-d.arg12004
                                                                                              :-d.arg22005
                                                                                              :-d.arg32006
                                                                                              :-d.arg42007)
                                                                                        :-d.body2010)
                                                                                  (list 'car
                                                                                        :-d.val2011)
                                                                                  (list 'cadr
                                                                                        :-d.val2011)
                                                                                  (list 'caddr
                                                                                        :-d.val2011)
                                                                                  (list 'cadddr
                                                                                        :-d.val2011)))
                                                                          (car :-d.y2009)
                                                                          (cadr :-d.y2009))
                                                                         (syntax-error
                                                                           :-d.x2008)))
                                                                     (syntax-dispatch
                                                                       :-d.x2008
                                                                       '(pair (any)
                                                                              pair
                                                                              (any)
                                                                              atom)
                                                                       '#())))
                                                                  (list :-d.body1984
                                                                        :-d.val1985)))
                                                               (car :-d.y1995)
                                                               (cadr :-d.y1995)
                                                               (caddr :-d.y1995)
                                                               (cadddr :-d.y1995))
                                                              ((lambda (:-d.x1996)
                                                                 ((lambda (:-d.y1997)
                                                                    (if :-d.y1997
                                                                      ((lambda (:-d.arg1998)
                                                                         ((lambda (:-d.x1999)
                                                                            ((lambda (:-d.y2000)
                                                                               (if :-d.y2000
                                                                                 ((lambda (:-d.body2001
                                                                                           :-d.val2002)
                                                                                    (list 'apply
                                                                                          (list 'syntax-lambda
                                                                                                :-d.arg1998
                                                                                                :-d.body2001)
                                                                                          :-d.val2002))
                                                                                  (car :-d.y2000)
                                                                                  (cadr :-d.y2000))
                                                                                 (syntax-error
                                                                                   :-d.x1999)))
                                                                             (syntax-dispatch
                                                                               :-d.x1999
                                                                               '(pair (any)
                                                                                      pair
                                                                                      (any)
                                                                                      atom)
                                                                               '#())))
                                                                          (list :-d.body1984
                                                                                :-d.val1985)))
                                                                       (car :-d.y1997))
                                                                      (syntax-error
                                                                        :-d.x1996)))
                                                                  (syntax-dispatch
                                                                    :-d.x1996
                                                                    '(each any)
                                                                    '#())))
                                                               :-d.x1994)))
                                                          (syntax-dispatch
                                                            :-d.x1994
                                                            '(pair (any)
                                                                   pair
                                                                   (any)
                                                                   pair
                                                                   (any)
                                                                   pair
                                                                   (any)
                                                                   atom)
                                                            '#())))
                                                       :-d.x1992)))
                                                  (syntax-dispatch
                                                    :-d.x1992
                                                    '(pair (any)
                                                           pair
                                                           (any)
                                                           pair
                                                           (any)
                                                           atom)
                                                    '#())))
                                               :-d.x1990)))
                                          (syntax-dispatch
                                            :-d.x1990
                                            '(pair (any) pair (any) atom)
                                            '#())))
                                       :-d.x1988)))
                                  (syntax-dispatch
                                    :-d.x1988
                                    '(pair (any) atom)
                                    '#())))
                               :-d.x1986)))
                          (syntax-dispatch :-d.x1986 '(atom) '#())))
                       :-d.args1983)))
                  (:-d.extract-bound-syntax-ids1914
                    (lambda (:-d.pattern2030 :-d.keys2031)
                      ((letrec ((:-d.gen2032
                                  (lambda (:-d.p2033 :-d.n2034 :-d.ids2035)
                                    (if (identifier? :-d.p2033)
                                      (if (:-d.key?1918 :-d.p2033 :-d.keys2031)
                                        :-d.ids2035
                                        (cons (list :-d.p2033 :-d.n2034)
                                              :-d.ids2035))
                                      ((lambda (:-d.x2036)
                                         ((lambda (:-d.dorest2048)
                                            ((lambda (:-d.y2049)
                                               (if :-d.y2049
                                                 ((lambda (:-d.x2050
                                                           :-d.dots2051)
                                                    (if (:-d.ellipsis?1920
                                                          :-d.dots2051
                                                          :-d.keys2031)
                                                      (:-d.gen2032
                                                        :-d.x2050
                                                        (+ :-d.n2034 '1)
                                                        :-d.ids2035)
                                                      (:-d.dorest2048)))
                                                  (car :-d.y2049)
                                                  (cadr :-d.y2049))
                                                 (:-d.dorest2048)))
                                             (syntax-dispatch
                                               :-d.x2036
                                               '(pair (any) pair (any) atom)
                                               '#())))
                                          (lambda ()
                                            ((lambda (:-d.x2037)
                                               ((lambda (:-d.y2038)
                                                  (if :-d.y2038
                                                    ((lambda (:-d.x2046
                                                              :-d.y2047)
                                                       (:-d.gen2032
                                                         :-d.x2046
                                                         :-d.n2034
                                                         (:-d.gen2032
                                                           :-d.y2047
                                                           :-d.n2034
                                                           :-d.ids2035)))
                                                     (car :-d.y2038)
                                                     (cadr :-d.y2038))
                                                    ((lambda (:-d.x2039)
                                                       ((lambda (:-d.y2040)
                                                          (if :-d.y2040
                                                            ((lambda (:-d.x2044)
                                                               (:-d.gen2032
                                                                 :-d.x2044
                                                                 :-d.n2034
                                                                 :-d.ids2035))
                                                             (car :-d.y2040))
                                                            ((lambda (:-d.x2041)
                                                               ((lambda (:-d.y2042)
                                                                  (if :-d.y2042
                                                                    ((lambda (:-d.x2043)
                                                                       :-d.ids2035)
                                                                     (car :-d.y2042))
                                                                    (syntax-error
                                                                      :-d.x2041)))
                                                                (syntax-dispatch
                                                                  :-d.x2041
                                                                  '(any)
                                                                  '#())))
                                                             :-d.x2039)))
                                                        (syntax-dispatch
                                                          :-d.x2039
                                                          '(vector each any)
                                                          '#())))
                                                     :-d.x2037)))
                                                (syntax-dispatch
                                                  :-d.x2037
                                                  '(pair (any) any)
                                                  '#())))
                                             :-d.x2036))))
                                       :-d.p2033)))))
                         :-d.gen2032)
                       :-d.pattern2030
                       '0
                       '())))
                  (:-d.valid-syntax-pattern?1915
                    (lambda (:-d.pattern2052 :-d.keys2053)
                      ((letrec ((:-d.check?2054
                                  (lambda (:-d.p2055 :-d.ids2056)
                                    (if (identifier? :-d.p2055)
                                      (if :-d.ids2056
                                        (if (:-d.key?1918
                                              :-d.p2055
                                              :-d.keys2053)
                                          :-d.ids2056
                                          (if (not (:-d.ellipsis?1920
                                                     :-d.p2055
                                                     :-d.keys2053))
                                            (if (not (:-d.memid1919
                                                       :-d.p2055
                                                       :-d.ids2056))
                                              (cons :-d.p2055 :-d.ids2056)
                                              '#f)
                                            '#f))
                                        '#f)
                                      ((lambda (:-d.x2057)
                                         ((lambda (:-d.dorest2069)
                                            ((lambda (:-d.y2070)
                                               (if :-d.y2070
                                                 ((lambda (:-d.x2071
                                                           :-d.dots2072)
                                                    (if (:-d.ellipsis?1920
                                                          :-d.dots2072
                                                          :-d.keys2053)
                                                      (:-d.check?2054
                                                        :-d.x2071
                                                        :-d.ids2056)
                                                      (:-d.dorest2069)))
                                                  (car :-d.y2070)
                                                  (cadr :-d.y2070))
                                                 (:-d.dorest2069)))
                                             (syntax-dispatch
                                               :-d.x2057
                                               '(pair (any) pair (any) atom)
                                               '#())))
                                          (lambda ()
                                            ((lambda (:-d.x2058)
                                               ((lambda (:-d.y2059)
                                                  (if :-d.y2059
                                                    ((lambda (:-d.x2067
                                                              :-d.y2068)
                                                       (:-d.check?2054
                                                         :-d.x2067
                                                         (:-d.check?2054
                                                           :-d.y2068
                                                           :-d.ids2056)))
                                                     (car :-d.y2059)
                                                     (cadr :-d.y2059))
                                                    ((lambda (:-d.x2060)
                                                       ((lambda (:-d.y2061)
                                                          (if :-d.y2061
                                                            ((lambda (:-d.x2065)
                                                               (:-d.check?2054
                                                                 :-d.x2065
                                                                 :-d.ids2056))
                                                             (car :-d.y2061))
                                                            ((lambda (:-d.x2062)
                                                               ((lambda (:-d.y2063)
                                                                  (if :-d.y2063
                                                                    ((lambda (:-d.x2064)
                                                                       :-d.ids2056)
                                                                     (car :-d.y2063))
                                                                    (syntax-error
                                                                      :-d.x2062)))
                                                                (syntax-dispatch
                                                                  :-d.x2062
                                                                  '(any)
                                                                  '#())))
                                                             :-d.x2060)))
                                                        (syntax-dispatch
                                                          :-d.x2060
                                                          '(vector each any)
                                                          '#())))
                                                     :-d.x2058)))
                                                (syntax-dispatch
                                                  :-d.x2058
                                                  '(pair (any) any)
                                                  '#())))
                                             :-d.x2057))))
                                       :-d.p2055)))))
                         :-d.check?2054)
                       :-d.pattern2052
                       '())))
                  (:-d.convert-syntax-dispatch-pattern1916
                    (lambda (:-d.pattern2073 :-d.keys2074)
                      ((letrec ((:-d.gen2075
                                  (lambda (:-d.p2076)
                                    (if (identifier? :-d.p2076)
                                      (if (:-d.key?1918 :-d.p2076 :-d.keys2074)
                                        (cons 'free-id
                                              (:-d.key-index1917
                                                :-d.p2076
                                                :-d.keys2074))
                                        '(any))
                                      ((lambda (:-d.x2077)
                                         ((lambda (:-d.dorest2089)
                                            ((lambda (:-d.y2090)
                                               (if :-d.y2090
                                                 ((lambda (:-d.x2091
                                                           :-d.dots2092)
                                                    (if (:-d.ellipsis?1920
                                                          :-d.dots2092
                                                          :-d.keys2074)
                                                      (cons 'each
                                                            (:-d.gen2075
                                                              :-d.x2091))
                                                      (:-d.dorest2089)))
                                                  (car :-d.y2090)
                                                  (cadr :-d.y2090))
                                                 (:-d.dorest2089)))
                                             (syntax-dispatch
                                               :-d.x2077
                                               '(pair (any) pair (any) atom)
                                               '#())))
                                          (lambda ()
                                            ((lambda (:-d.x2078)
                                               ((lambda (:-d.y2079)
                                                  (if :-d.y2079
                                                    ((lambda (:-d.x2087
                                                              :-d.y2088)
                                                       (cons 'pair
                                                             (cons (:-d.gen2075
                                                                     :-d.x2087)
                                                                   (:-d.gen2075
                                                                     :-d.y2088))))
                                                     (car :-d.y2079)
                                                     (cadr :-d.y2079))
                                                    ((lambda (:-d.x2080)
                                                       ((lambda (:-d.y2081)
                                                          (if :-d.y2081
                                                            ((lambda (:-d.x2085)
                                                               (cons 'vector
                                                                     (:-d.gen2075
                                                                       :-d.x2085)))
                                                             (car :-d.y2081))
                                                            ((lambda (:-d.x2082)
                                                               ((lambda (:-d.y2083)
                                                                  (if :-d.y2083
                                                                    ((lambda (:-d.x2084)
                                                                       (cons 'atom
                                                                             :-d.p2076))
                                                                     (car :-d.y2083))
                                                                    (syntax-error
                                                                      :-d.x2082)))
                                                                (syntax-dispatch
                                                                  :-d.x2082
                                                                  '(any)
                                                                  '#())))
                                                             :-d.x2080)))
                                                        (syntax-dispatch
                                                          :-d.x2080
                                                          '(vector each any)
                                                          '#())))
                                                     :-d.x2078)))
                                                (syntax-dispatch
                                                  :-d.x2078
                                                  '(pair (any) any)
                                                  '#())))
                                             :-d.x2077))))
                                       :-d.p2076)))))
                         :-d.gen2075)
                       :-d.pattern2073)))
                  (:-d.key-index1917
                    (lambda (:-d.p2093 :-d.keys2094)
                      (- (length :-d.keys2094)
                         (length (:-d.memid1919 :-d.p2093 :-d.keys2094)))))
                  (:-d.key?1918
                    (lambda (:-d.p2095 :-d.keys2096)
                      (if (identifier? :-d.p2095)
                        (:-d.memid1919 :-d.p2095 :-d.keys2096)
                        '#f)))
                  (:-d.memid1919
                    (lambda (:-d.i2097 :-d.ids2098)
                      (if (not (null? :-d.ids2098))
                        (if (bound-identifier=? :-d.i2097 (car :-d.ids2098))
                          :-d.ids2098
                          (:-d.memid1919 :-d.i2097 (cdr :-d.ids2098)))
                        '#f)))
                  (:-d.ellipsis?1920
                    (lambda (:-d.i2099 :-d.keys2100)
                      (if (identifier? :-d.i2099)
                        (if (free-identifier=? :-d.i2099 '...)
                          (not (:-d.key?1918 :-d.i2099 :-d.keys2100))
                          '#f)
                        '#f))))
           (lambda (:-d.x1921)
             ((lambda (:-d.x1922)
                ((lambda (:-d.dorest1977)
                   ((lambda (:-d.y1978)
                      (if :-d.y1978
                        ((lambda (:-d._1979 :-d.val1980 :-d.key1981)
                           (if (andmap identifier? :-d.key1981)
                             (list 'syntax-error :-d.val1980)
                             (:-d.dorest1977)))
                         (car :-d.y1978)
                         (cadr :-d.y1978)
                         (caddr :-d.y1978))
                        (:-d.dorest1977)))
                    (syntax-dispatch
                      :-d.x1922
                      '(pair (any) pair (any) pair (each any) atom)
                      '#())))
                 (lambda ()
                   ((lambda (:-d.x1923)
                      ((lambda (:-d.dorest1952)
                         ((lambda (:-d.y1953)
                            (if :-d.y1953
                              (apply (lambda (:-d._1954
                                              :-d.val1955
                                              :-d.key1956
                                              :-d.pat1957
                                              :-d.exp1958
                                              :-d.e11959
                                              :-d.e21960
                                              :-d.e31961)
                                       (if (if (andmap identifier? :-d.key1956)
                                             (:-d.valid-syntax-pattern?1915
                                               :-d.pat1957
                                               :-d.key1956)
                                             '#f)
                                         ((lambda (:-d.x1964)
                                            ((lambda (:-d.y1965)
                                               (if :-d.y1965
                                                 ((lambda (:-d.pattern1966
                                                           :-d.y1967
                                                           :-d.call1968)
                                                    (list 'let
                                                          (list (list 'x
                                                                      :-d.val1955))
                                                          (list 'let
                                                                (list (list :-d.y1967
                                                                            (list 'syntax-dispatch
                                                                                  'x
                                                                                  (list 'quote
                                                                                        :-d.pattern1966)
                                                                                  (list 'syntax
                                                                                        (list->vector
                                                                                          :-d.key1956)))))
                                                                (list 'if
                                                                      :-d.y1967
                                                                      :-d.call1968
                                                                      (cons 'syntax-case
                                                                            (cons 'x
                                                                                  (cons :-d.key1956
                                                                                        (map (lambda (:-d.:-d.e319611973
                                                                                                      :-d.:-d.e219601972
                                                                                                      :-d.:-d.e119591971)
                                                                                               (cons :-d.:-d.e119591971
                                                                                                     (cons :-d.:-d.e219601972
                                                                                                           :-d.:-d.e319611973)))
                                                                                             :-d.e31961
                                                                                             :-d.e21960
                                                                                             :-d.e11959))))))))
                                                  (car :-d.y1965)
                                                  (cadr :-d.y1965)
                                                  (caddr :-d.y1965))
                                                 (syntax-error :-d.x1964)))
                                             (syntax-dispatch
                                               :-d.x1964
                                               '(pair (any)
                                                      pair
                                                      (any)
                                                      pair
                                                      (any)
                                                      atom)
                                               '#())))
                                          (list (:-d.convert-syntax-dispatch-pattern1916
                                                  :-d.pat1957
                                                  :-d.key1956)
                                                'y
                                                (:-d.build-dispatch-call1913
                                                  (:-d.extract-bound-syntax-ids1914
                                                    :-d.pat1957
                                                    :-d.key1956)
                                                  :-d.exp1958
                                                  'y)))
                                         (:-d.dorest1952)))
                                     :-d.y1953)
                              (:-d.dorest1952)))
                          (syntax-dispatch
                            :-d.x1923
                            '(pair (any)
                                   pair
                                   (any)
                                   pair
                                   (each any)
                                   pair
                                   (pair (any) pair (any) atom)
                                   each
                                   pair
                                   (any)
                                   pair
                                   (any)
                                   each
                                   any)
                            '#())))
                       (lambda ()
                         ((lambda (:-d.x1924)
                            ((lambda (:-d.dorest1925)
                               ((lambda (:-d.y1926)
                                  (if :-d.y1926
                                    (apply (lambda (:-d._1927
                                                    :-d.val1928
                                                    :-d.key1929
                                                    :-d.pat1930
                                                    :-d.fender1931
                                                    :-d.exp1932
                                                    :-d.e11933
                                                    :-d.e21934
                                                    :-d.e31935)
                                             (if (if (andmap identifier?
                                                             :-d.key1929)
                                                   (:-d.valid-syntax-pattern?1915
                                                     :-d.pat1930
                                                     :-d.key1929)
                                                   '#f)
                                               ((lambda (:-d.x1938)
                                                  ((lambda (:-d.y1939)
                                                     (if :-d.y1939
                                                       ((lambda (:-d.pattern1940
                                                                 :-d.y1941
                                                                 :-d.dorest1942
                                                                 :-d.call1943)
                                                          (list 'let
                                                                (list (list 'x
                                                                            :-d.val1928))
                                                                (list 'let
                                                                      (list (list :-d.dorest1942
                                                                                  (list 'lambda
                                                                                        '()
                                                                                        (cons 'syntax-case
                                                                                              (cons 'x
                                                                                                    (cons :-d.key1929
                                                                                                          (map (lambda (:-d.:-d.e319351947
                                                                                                                        :-d.:-d.e219341946
                                                                                                                        :-d.:-d.e119331945)
                                                                                                                 (cons :-d.:-d.e119331945
                                                                                                                       (cons :-d.:-d.e219341946
                                                                                                                             :-d.:-d.e319351947)))
                                                                                                               :-d.e31935
                                                                                                               :-d.e21934
                                                                                                               :-d.e11933)))))))
                                                                      (list 'let
                                                                            (list (list :-d.y1941
                                                                                        (list 'syntax-dispatch
                                                                                              'x
                                                                                              (list 'quote
                                                                                                    :-d.pattern1940)
                                                                                              (list 'syntax
                                                                                                    (list->vector
                                                                                                      :-d.key1929)))))
                                                                            (list 'if
                                                                                  :-d.y1941
                                                                                  :-d.call1943
                                                                                  (list :-d.dorest1942))))))
                                                        (car :-d.y1939)
                                                        (cadr :-d.y1939)
                                                        (caddr :-d.y1939)
                                                        (cadddr :-d.y1939))
                                                       (syntax-error
                                                         :-d.x1938)))
                                                   (syntax-dispatch
                                                     :-d.x1938
                                                     '(pair (any)
                                                            pair
                                                            (any)
                                                            pair
                                                            (any)
                                                            pair
                                                            (any)
                                                            atom)
                                                     '#())))
                                                (list (:-d.convert-syntax-dispatch-pattern1916
                                                        :-d.pat1930
                                                        :-d.key1929)
                                                      'y
                                                      'dorest
                                                      (:-d.build-dispatch-call1913
                                                        (:-d.extract-bound-syntax-ids1914
                                                          :-d.pat1930
                                                          :-d.key1929)
                                                        (list 'if
                                                              :-d.fender1931
                                                              :-d.exp1932
                                                              (list 'dorest))
                                                        'y)))
                                               (:-d.dorest1925)))
                                           :-d.y1926)
                                    (:-d.dorest1925)))
                                (syntax-dispatch
                                  :-d.x1924
                                  '(pair (any)
                                         pair
                                         (any)
                                         pair
                                         (each any)
                                         pair
                                         (pair (any)
                                               pair
                                               (any)
                                               pair
                                               (any)
                                               atom)
                                         each
                                         pair
                                         (any)
                                         pair
                                         (any)
                                         each
                                         any)
                                  '#())))
                             (lambda () (syntax-error :-d.x1924))))
                          :-d.x1923))))
                    :-d.x1922))))
              :-d.x1921))))))))

;------------------------------------------------------------------------------

; From file "macro-defs.bb".

(install-global-transformer
  'with-syntax
  (lambda (:-d.x2101)
    ((lambda (:-d.x2102)
       ((lambda (:-d.y2103)
          (if :-d.y2103
            ((lambda (:-d._2122 :-d.e12123 :-d.e22124)
               (cons 'begin (cons :-d.e12123 :-d.e22124)))
             (car :-d.y2103)
             (cadr :-d.y2103)
             (caddr :-d.y2103))
            ((lambda (:-d.x2104)
               ((lambda (:-d.y2105)
                  (if :-d.y2105
                    (apply (lambda (:-d._2116
                                    :-d.out2117
                                    :-d.in2118
                                    :-d.e12119
                                    :-d.e22120)
                             (list 'syntax-case
                                   :-d.in2118
                                   '()
                                   (list :-d.out2117
                                         (cons 'begin
                                               (cons :-d.e12119 :-d.e22120)))))
                           :-d.y2105)
                    ((lambda (:-d.x2106)
                       ((lambda (:-d.y2107)
                          (if :-d.y2107
                            (apply (lambda (:-d._2108
                                            :-d.out2109
                                            :-d.in2110
                                            :-d.e12111
                                            :-d.e22112)
                                     (list 'syntax-case
                                           (cons 'list :-d.in2110)
                                           '()
                                           (list :-d.out2109
                                                 (cons 'begin
                                                       (cons :-d.e12111
                                                             :-d.e22112)))))
                                   :-d.y2107)
                            (syntax-error :-d.x2106)))
                        (syntax-dispatch
                          :-d.x2106
                          '(pair (any)
                                 pair
                                 (each pair (any) pair (any) atom)
                                 pair
                                 (any)
                                 each
                                 any)
                          '#())))
                     :-d.x2104)))
                (syntax-dispatch
                  :-d.x2104
                  '(pair (any)
                         pair
                         (pair (pair (any) pair (any) atom) atom)
                         pair
                         (any)
                         each
                         any)
                  '#())))
             :-d.x2102)))
        (syntax-dispatch
          :-d.x2102
          '(pair (any) pair (atom) pair (any) each any)
          '#())))
     :-d.x2101)))

(install-global-transformer
  'syntax-rules
  (lambda (:-d.x2126)
    ((lambda (:-d.x2127)
       ((lambda (:-d.y2128)
          (if :-d.y2128
            (apply (lambda (:-d._2129
                            :-d.k2130
                            :-d.keyword2131
                            :-d.pattern2132
                            :-d.template2133)
                     ((lambda (:-d.x2135)
                        ((lambda (:-d.y2136)
                           (if :-d.y2136
                             ((lambda (:-d.dummy2137)
                                (list 'lambda
                                      (list 'x)
                                      (cons 'syntax-case
                                            (cons 'x
                                                  (cons :-d.k2130
                                                        (map (lambda (:-d.:-d.template21332141
                                                                      :-d.:-d.pattern21322140
                                                                      :-d.:-d.dummy21372139)
                                                               (list (cons :-d.:-d.dummy21372139
                                                                           :-d.:-d.pattern21322140)
                                                                     (list 'syntax
                                                                           :-d.:-d.template21332141)))
                                                             :-d.template2133
                                                             :-d.pattern2132
                                                             :-d.dummy2137))))))
                              (car :-d.y2136))
                             (syntax-error :-d.x2135)))
                         (syntax-dispatch :-d.x2135 '(each any) '#())))
                      (generate-temporaries :-d.keyword2131)))
                   :-d.y2128)
            (syntax-error :-d.x2127)))
        (syntax-dispatch
          :-d.x2127
          '(pair (any)
                 pair
                 (each any)
                 each
                 pair
                 (pair (any) any)
                 pair
                 (any)
                 atom)
          '#())))
     :-d.x2126)))

(install-global-transformer
  'or
  (lambda (:-d.x2142)
    ((lambda (:-d.x2143)
       ((lambda (:-d.y2144)
          (if :-d.y2144
            ((lambda (:-d._2156) '#f) (car :-d.y2144))
            ((lambda (:-d.x2145)
               ((lambda (:-d.y2146)
                  (if :-d.y2146
                    ((lambda (:-d._2154 :-d.e2155) :-d.e2155)
                     (car :-d.y2146)
                     (cadr :-d.y2146))
                    ((lambda (:-d.x2147)
                       ((lambda (:-d.y2148)
                          (if :-d.y2148
                            ((lambda (:-d._2149
                                      :-d.e12150
                                      :-d.e22151
                                      :-d.e32152)
                               (list 'let
                                     (list (list 't :-d.e12150))
                                     (list 'if
                                           't
                                           't
                                           (cons 'or
                                                 (cons :-d.e22151
                                                       :-d.e32152)))))
                             (car :-d.y2148)
                             (cadr :-d.y2148)
                             (caddr :-d.y2148)
                             (cadddr :-d.y2148))
                            (syntax-error :-d.x2147)))
                        (syntax-dispatch
                          :-d.x2147
                          '(pair (any) pair (any) pair (any) each any)
                          '#())))
                     :-d.x2145)))
                (syntax-dispatch
                  :-d.x2145
                  '(pair (any) pair (any) atom)
                  '#())))
             :-d.x2143)))
        (syntax-dispatch :-d.x2143 '(pair (any) atom) '#())))
     :-d.x2142)))

(install-global-transformer
  'and
  (lambda (:-d.x2157)
    ((lambda (:-d.x2158)
       ((lambda (:-d.y2159)
          (if :-d.y2159
            ((lambda (:-d._2167 :-d.e12168 :-d.e22169 :-d.e32170)
               (cons 'if
                     (cons :-d.e12168
                           (cons (cons 'and (cons :-d.e22169 :-d.e32170))
                                 '(#f)))))
             (car :-d.y2159)
             (cadr :-d.y2159)
             (caddr :-d.y2159)
             (cadddr :-d.y2159))
            ((lambda (:-d.x2160)
               ((lambda (:-d.y2161)
                  (if :-d.y2161
                    ((lambda (:-d._2165 :-d.e2166) :-d.e2166)
                     (car :-d.y2161)
                     (cadr :-d.y2161))
                    ((lambda (:-d.x2162)
                       ((lambda (:-d.y2163)
                          (if :-d.y2163
                            ((lambda (:-d._2164) '#t) (car :-d.y2163))
                            (syntax-error :-d.x2162)))
                        (syntax-dispatch :-d.x2162 '(pair (any) atom) '#())))
                     :-d.x2160)))
                (syntax-dispatch
                  :-d.x2160
                  '(pair (any) pair (any) atom)
                  '#())))
             :-d.x2158)))
        (syntax-dispatch
          :-d.x2158
          '(pair (any) pair (any) pair (any) each any)
          '#())))
     :-d.x2157)))

(install-global-transformer
  'cond
  (lambda (:-d.x2172)
    ((lambda (:-d.x2173)
       ((lambda (:-d.y2174)
          (if :-d.y2174
            ((lambda (:-d._2216 :-d.e12217 :-d.e22218)
               (cons 'begin (cons :-d.e12217 :-d.e22218)))
             (car :-d.y2174)
             (cadr :-d.y2174)
             (caddr :-d.y2174))
            ((lambda (:-d.x2175)
               ((lambda (:-d.y2176)
                  (if :-d.y2176
                    ((lambda (:-d._2214 :-d.e02215)
                       (list 'let
                             (list (list 't :-d.e02215))
                             (list 'if 't 't)))
                     (car :-d.y2176)
                     (cadr :-d.y2176))
                    ((lambda (:-d.x2177)
                       ((lambda (:-d.y2178)
                          (if :-d.y2178
                            ((lambda (:-d._2209
                                      :-d.e02210
                                      :-d.c12211
                                      :-d.c22212)
                               (list 'let
                                     (list (list 't :-d.e02210))
                                     (list 'if
                                           't
                                           't
                                           (cons 'cond
                                                 (cons :-d.c12211
                                                       :-d.c22212)))))
                             (car :-d.y2178)
                             (cadr :-d.y2178)
                             (caddr :-d.y2178)
                             (cadddr :-d.y2178))
                            ((lambda (:-d.x2179)
                               ((lambda (:-d.y2180)
                                  (if :-d.y2180
                                    ((lambda (:-d._2206 :-d.e02207 :-d.e12208)
                                       (list 'let
                                             (list (list 't :-d.e02207))
                                             (list 'if
                                                   't
                                                   (list :-d.e12208 't))))
                                     (car :-d.y2180)
                                     (cadr :-d.y2180)
                                     (caddr :-d.y2180))
                                    ((lambda (:-d.x2181)
                                       ((lambda (:-d.y2182)
                                          (if :-d.y2182
                                            (apply (lambda (:-d._2200
                                                            :-d.e02201
                                                            :-d.e12202
                                                            :-d.c12203
                                                            :-d.c22204)
                                                     (list 'let
                                                           (list (list 't
                                                                       :-d.e02201))
                                                           (list 'if
                                                                 't
                                                                 (list :-d.e12202
                                                                       't)
                                                                 (cons 'cond
                                                                       (cons :-d.c12203
                                                                             :-d.c22204)))))
                                                   :-d.y2182)
                                            ((lambda (:-d.x2183)
                                               ((lambda (:-d.y2184)
                                                  (if :-d.y2184
                                                    ((lambda (:-d._2195
                                                              :-d.e02196
                                                              :-d.e12197
                                                              :-d.e22198)
                                                       (list 'if
                                                             :-d.e02196
                                                             (cons 'begin
                                                                   (cons :-d.e12197
                                                                         :-d.e22198))))
                                                     (car :-d.y2184)
                                                     (cadr :-d.y2184)
                                                     (caddr :-d.y2184)
                                                     (cadddr :-d.y2184))
                                                    ((lambda (:-d.x2185)
                                                       ((lambda (:-d.y2186)
                                                          (if :-d.y2186
                                                            (apply (lambda (:-d._2187
                                                                            :-d.e02188
                                                                            :-d.e12189
                                                                            :-d.e22190
                                                                            :-d.c12191
                                                                            :-d.c22192)
                                                                     (list 'if
                                                                           :-d.e02188
                                                                           (cons 'begin
                                                                                 (cons :-d.e12189
                                                                                       :-d.e22190))
                                                                           (cons 'cond
                                                                                 (cons :-d.c12191
                                                                                       :-d.c22192))))
                                                                   :-d.y2186)
                                                            (syntax-error
                                                              :-d.x2185)))
                                                        (syntax-dispatch
                                                          :-d.x2185
                                                          '(pair (any)
                                                                 pair
                                                                 (pair (any)
                                                                       pair
                                                                       (any)
                                                                       each
                                                                       any)
                                                                 pair
                                                                 (any)
                                                                 each
                                                                 any)
                                                          (vector 'else '=>))))
                                                     :-d.x2183)))
                                                (syntax-dispatch
                                                  :-d.x2183
                                                  '(pair (any)
                                                         pair
                                                         (pair (any)
                                                               pair
                                                               (any)
                                                               each
                                                               any)
                                                         atom)
                                                  (vector 'else '=>))))
                                             :-d.x2181)))
                                        (syntax-dispatch
                                          :-d.x2181
                                          '(pair (any)
                                                 pair
                                                 (pair (any)
                                                       pair
                                                       (free-id . 1)
                                                       pair
                                                       (any)
                                                       atom)
                                                 pair
                                                 (any)
                                                 each
                                                 any)
                                          (vector 'else '=>))))
                                     :-d.x2179)))
                                (syntax-dispatch
                                  :-d.x2179
                                  '(pair (any)
                                         pair
                                         (pair (any)
                                               pair
                                               (free-id . 1)
                                               pair
                                               (any)
                                               atom)
                                         atom)
                                  (vector 'else '=>))))
                             :-d.x2177)))
                        (syntax-dispatch
                          :-d.x2177
                          '(pair (any)
                                 pair
                                 (pair (any) atom)
                                 pair
                                 (any)
                                 each
                                 any)
                          (vector 'else '=>))))
                     :-d.x2175)))
                (syntax-dispatch
                  :-d.x2175
                  '(pair (any) pair (pair (any) atom) atom)
                  (vector 'else '=>))))
             :-d.x2173)))
        (syntax-dispatch
          :-d.x2173
          '(pair (any) pair (pair (free-id . 0) pair (any) each any) atom)
          (vector 'else '=>))))
     :-d.x2172)))

(install-global-transformer
  'let*
  (lambda (:-d.x2220)
    ((lambda (:-d.x2221)
       ((lambda (:-d.y2222)
          (if :-d.y2222
            ((lambda (:-d.let*2237 :-d.e12238 :-d.e22239)
               (cons 'let (cons '() (cons :-d.e12238 :-d.e22239))))
             (car :-d.y2222)
             (cadr :-d.y2222)
             (caddr :-d.y2222))
            ((lambda (:-d.x2223)
               ((lambda (:-d.dorest2224)
                  ((lambda (:-d.y2225)
                     (if :-d.y2225
                       (apply (lambda (:-d.let*2226
                                       :-d.x12227
                                       :-d.v12228
                                       :-d.x22229
                                       :-d.v22230
                                       :-d.e12231
                                       :-d.e22232)
                                (if (andmap identifier?
                                            (cons :-d.x12227 :-d.x22229))
                                  (list 'let
                                        (list (list :-d.x12227 :-d.v12228))
                                        (cons :-d.let*2226
                                              (cons (map list
                                                         :-d.x22229
                                                         :-d.v22230)
                                                    (cons :-d.e12231
                                                          :-d.e22232))))
                                  (:-d.dorest2224)))
                              :-d.y2225)
                       (:-d.dorest2224)))
                   (syntax-dispatch
                     :-d.x2223
                     '(pair (any)
                            pair
                            (pair (pair (any) pair (any) atom)
                                  each
                                  pair
                                  (any)
                                  pair
                                  (any)
                                  atom)
                            pair
                            (any)
                            each
                            any)
                     '#())))
                (lambda () (syntax-error :-d.x2223))))
             :-d.x2221)))
        (syntax-dispatch
          :-d.x2221
          '(pair (any) pair (atom) pair (any) each any)
          '#())))
     :-d.x2220)))

(install-global-transformer
  'case
  (lambda (:-d.x2241)
    ((lambda (:-d.x2242)
       ((lambda (:-d.y2243)
          (if :-d.y2243
            ((lambda (:-d._2265 :-d.v2266 :-d.e12267 :-d.e22268)
               (cons 'begin (cons :-d.v2266 (cons :-d.e12267 :-d.e22268))))
             (car :-d.y2243)
             (cadr :-d.y2243)
             (caddr :-d.y2243)
             (cadddr :-d.y2243))
            ((lambda (:-d.x2244)
               ((lambda (:-d.y2245)
                  (if :-d.y2245
                    (apply (lambda (:-d._2258
                                    :-d.v2259
                                    :-d.k2260
                                    :-d.e12261
                                    :-d.e22262)
                             (list 'if
                                   (list 'memv
                                         :-d.v2259
                                         (list 'quote :-d.k2260))
                                   (cons 'begin (cons :-d.e12261 :-d.e22262))))
                           :-d.y2245)
                    ((lambda (:-d.x2246)
                       ((lambda (:-d.y2247)
                          (if :-d.y2247
                            (apply (lambda (:-d._2248
                                            :-d.v2249
                                            :-d.k2250
                                            :-d.e12251
                                            :-d.e22252
                                            :-d.c12253
                                            :-d.c22254)
                                     (list 'let
                                           (list (list 'x :-d.v2249))
                                           (list 'if
                                                 (list 'memv
                                                       'x
                                                       (list 'quote :-d.k2250))
                                                 (cons 'begin
                                                       (cons :-d.e12251
                                                             :-d.e22252))
                                                 (cons 'case
                                                       (cons 'x
                                                             (cons :-d.c12253
                                                                   :-d.c22254))))))
                                   :-d.y2247)
                            (syntax-error :-d.x2246)))
                        (syntax-dispatch
                          :-d.x2246
                          '(pair (any)
                                 pair
                                 (any)
                                 pair
                                 (pair (each any) pair (any) each any)
                                 pair
                                 (any)
                                 each
                                 any)
                          (vector 'else))))
                     :-d.x2244)))
                (syntax-dispatch
                  :-d.x2244
                  '(pair (any)
                         pair
                         (any)
                         pair
                         (pair (each any) pair (any) each any)
                         atom)
                  (vector 'else))))
             :-d.x2242)))
        (syntax-dispatch
          :-d.x2242
          '(pair (any)
                 pair
                 (any)
                 pair
                 (pair (free-id . 0) pair (any) each any)
                 atom)
          (vector 'else))))
     :-d.x2241)))

(install-global-transformer
  'do
  (lambda (:-d.orig-x2270)
    ((lambda (:-d.x2271)
       ((lambda (:-d.y2272)
          (if :-d.y2272
            (apply (lambda (:-d._2273
                            :-d.var2274
                            :-d.init2275
                            :-d.step2276
                            :-d.e02277
                            :-d.e12278
                            :-d.c2279)
                     ((lambda (:-d.x2292)
                        ((lambda (:-d.y2293)
                           (if :-d.y2293
                             ((lambda (:-d.step2294)
                                ((lambda (:-d.x2296)
                                   ((lambda (:-d.y2297)
                                      (if :-d.y2297
                                        (list 'let
                                              'doloop
                                              (map list
                                                   :-d.var2274
                                                   :-d.init2275)
                                              (list 'if
                                                    (list 'not :-d.e02277)
                                                    (cons 'begin
                                                          (append :-d.c2279
                                                                  (list (cons 'doloop
                                                                              :-d.step2294))))))
                                        ((lambda (:-d.x2298)
                                           ((lambda (:-d.y2299)
                                              (if :-d.y2299
                                                ((lambda (:-d.e2300)
                                                   (list 'let
                                                         'doloop
                                                         (map list
                                                              :-d.var2274
                                                              :-d.init2275)
                                                         (list 'if
                                                               :-d.e02277
                                                               :-d.e2300
                                                               (cons 'begin
                                                                     (append :-d.c2279
                                                                             (list (cons 'doloop
                                                                                         :-d.step2294)))))))
                                                 (car :-d.y2299))
                                                (syntax-error :-d.x2298)))
                                            (syntax-dispatch
                                              :-d.x2298
                                              '(pair (any) atom)
                                              '#())))
                                         :-d.x2296)))
                                    (syntax-dispatch :-d.x2296 '(atom) '#())))
                                 :-d.e12278))
                              (car :-d.y2293))
                             (syntax-error :-d.x2292)))
                         (syntax-dispatch :-d.x2292 '(each any) '#())))
                      (map (lambda (:-d.v2282 :-d.s2283)
                             ((lambda (:-d.x2284)
                                ((lambda (:-d.y2285)
                                   (if :-d.y2285
                                     :-d.v2282
                                     ((lambda (:-d.x2286)
                                        ((lambda (:-d.y2287)
                                           (if :-d.y2287
                                             ((lambda (:-d.e2291) :-d.e2291)
                                              (car :-d.y2287))
                                             ((lambda (:-d.x2288)
                                                ((lambda (:-d.y2289)
                                                   (if :-d.y2289
                                                     ((lambda (:-d._2290)
                                                        (syntax-error
                                                          :-d.orig-x2270))
                                                      (car :-d.y2289))
                                                     (syntax-error :-d.x2288)))
                                                 (syntax-dispatch
                                                   :-d.x2288
                                                   '(any)
                                                   '#())))
                                              :-d.x2286)))
                                         (syntax-dispatch
                                           :-d.x2286
                                           '(pair (any) atom)
                                           '#())))
                                      :-d.x2284)))
                                 (syntax-dispatch :-d.x2284 '(atom) '#())))
                              :-d.s2283))
                           :-d.var2274
                           :-d.step2276)))
                   :-d.y2272)
            (syntax-error :-d.x2271)))
        (syntax-dispatch
          :-d.x2271
          '(pair (any)
                 pair
                 (each pair (any) pair (any) any)
                 pair
                 (pair (any) each any)
                 each
                 any)
          '#())))
     :-d.orig-x2270)))

(install-global-transformer
  'quasiquote
  (letrec ((:-d.gen-cons2309
             (lambda (:-d.x2318 :-d.y2319)
               ((lambda (:-d.x2320)
                  ((lambda (:-d.y2321)
                     (if :-d.y2321
                       ((lambda (:-d.x2334)
                          ((lambda (:-d.x2335)
                             ((lambda (:-d.y2336)
                                (if :-d.y2336
                                  ((lambda (:-d.y2344)
                                     (list 'quote (cons :-d.x2334 :-d.y2344)))
                                   (car :-d.y2336))
                                  ((lambda (:-d.x2337)
                                     ((lambda (:-d.y2338)
                                        (if :-d.y2338
                                          ((lambda (:-d.y2342)
                                             (cons 'list
                                                   (cons (list 'quote
                                                               :-d.x2334)
                                                         :-d.y2342)))
                                           (car :-d.y2338))
                                          ((lambda (:-d.x2339)
                                             ((lambda (:-d.y2340)
                                                (if :-d.y2340
                                                  ((lambda (:-d.y2341)
                                                     (list 'cons
                                                           (list 'quote
                                                                 :-d.x2334)
                                                           :-d.y2341))
                                                   (car :-d.y2340))
                                                  (syntax-error :-d.x2339)))
                                              (syntax-dispatch
                                                :-d.x2339
                                                '(any)
                                                (vector 'quote 'list))))
                                           :-d.x2337)))
                                      (syntax-dispatch
                                        :-d.x2337
                                        '(pair (free-id . 1) each any)
                                        (vector 'quote 'list))))
                                   :-d.x2335)))
                              (syntax-dispatch
                                :-d.x2335
                                '(pair (free-id . 0) pair (any) atom)
                                (vector 'quote 'list))))
                           :-d.y2319))
                        (car :-d.y2321))
                       ((lambda (:-d.x2322)
                          ((lambda (:-d.y2323)
                             (if :-d.y2323
                               ((lambda (:-d.x2324)
                                  ((lambda (:-d.x2325)
                                     ((lambda (:-d.y2326)
                                        (if :-d.y2326
                                          (list 'list :-d.x2324)
                                          ((lambda (:-d.x2327)
                                             ((lambda (:-d.y2328)
                                                (if :-d.y2328
                                                  ((lambda (:-d.y2332)
                                                     (cons 'list
                                                           (cons :-d.x2324
                                                                 :-d.y2332)))
                                                   (car :-d.y2328))
                                                  ((lambda (:-d.x2329)
                                                     ((lambda (:-d.y2330)
                                                        (if :-d.y2330
                                                          ((lambda (:-d.y2331)
                                                             (list 'cons
                                                                   :-d.x2324
                                                                   :-d.y2331))
                                                           (car :-d.y2330))
                                                          (syntax-error
                                                            :-d.x2329)))
                                                      (syntax-dispatch
                                                        :-d.x2329
                                                        '(any)
                                                        (vector 'quote
                                                                'list))))
                                                   :-d.x2327)))
                                              (syntax-dispatch
                                                :-d.x2327
                                                '(pair (free-id . 1) each any)
                                                (vector 'quote 'list))))
                                           :-d.x2325)))
                                      (syntax-dispatch
                                        :-d.x2325
                                        '(pair (free-id . 0) pair (atom) atom)
                                        (vector 'quote 'list))))
                                   :-d.y2319))
                                (car :-d.y2323))
                               (syntax-error :-d.x2322)))
                           (syntax-dispatch :-d.x2322 '(any) (vector 'quote))))
                        :-d.x2320)))
                   (syntax-dispatch
                     :-d.x2320
                     '(pair (free-id . 0) pair (any) atom)
                     (vector 'quote))))
                :-d.x2318)))
           (:-d.gen-append2310
             (lambda (:-d.x2345 :-d.y2346)
               ((lambda (:-d.x2347)
                  ((lambda (:-d.y2348)
                     (if :-d.y2348
                       ((lambda (:-d.x12364 :-d.x22365)
                          ((lambda (:-d.x2366)
                             ((lambda (:-d.y2367)
                                (if :-d.y2367
                                  ((lambda (:-d.y2372)
                                     (list 'quote
                                           (cons :-d.x12364
                                                 (append :-d.x22365
                                                         :-d.y2372))))
                                   (car :-d.y2367))
                                  ((lambda (:-d.x2368)
                                     ((lambda (:-d.y2369)
                                        (if :-d.y2369
                                          ((lambda (:-d.y2370)
                                             (list 'append
                                                   (list 'quote
                                                         (cons :-d.x12364
                                                               :-d.x22365)
                                                         :-d.y2370)))
                                           (car :-d.y2369))
                                          (syntax-error :-d.x2368)))
                                      (syntax-dispatch
                                        :-d.x2368
                                        '(any)
                                        (vector 'quote))))
                                   :-d.x2366)))
                              (syntax-dispatch
                                :-d.x2366
                                '(pair (free-id . 0) pair (any) atom)
                                (vector 'quote))))
                           :-d.y2346))
                        (car :-d.y2348)
                        (cadr :-d.y2348))
                       ((lambda (:-d.x2349)
                          ((lambda (:-d.y2350)
                             (if :-d.y2350
                               :-d.y2346
                               ((lambda (:-d.x2351)
                                  ((lambda (:-d.y2352)
                                     (if :-d.y2352
                                       ((lambda (:-d.x12361 :-d.x22362)
                                          (:-d.gen-cons2309
                                            :-d.x12361
                                            (:-d.gen-append2310
                                              (cons 'list :-d.x22362)
                                              :-d.y2346)))
                                        (car :-d.y2352)
                                        (cadr :-d.y2352))
                                       ((lambda (:-d.x2353)
                                          ((lambda (:-d.y2354)
                                             (if :-d.y2354
                                               ((lambda (:-d.x2355)
                                                  ((lambda (:-d.x2356)
                                                     ((lambda (:-d.y2357)
                                                        (if :-d.y2357
                                                          :-d.x2355
                                                          ((lambda (:-d.x2358)
                                                             ((lambda (:-d.y2359)
                                                                (if :-d.y2359
                                                                  ((lambda (:-d.y2360)
                                                                     (list 'append
                                                                           :-d.x2355
                                                                           :-d.y2360))
                                                                   (car :-d.y2359))
                                                                  (syntax-error
                                                                    :-d.x2358)))
                                                              (syntax-dispatch
                                                                :-d.x2358
                                                                '(any)
                                                                (vector 'quote
                                                                        'list))))
                                                           :-d.x2356)))
                                                      (syntax-dispatch
                                                        :-d.x2356
                                                        '(pair (free-id . 0)
                                                               pair
                                                               (atom)
                                                               atom)
                                                        (vector 'quote
                                                                'list))))
                                                   :-d.y2346))
                                                (car :-d.y2354))
                                               (syntax-error :-d.x2353)))
                                           (syntax-dispatch
                                             :-d.x2353
                                             '(any)
                                             (vector 'quote 'list 'cons))))
                                        :-d.x2351)))
                                   (syntax-dispatch
                                     :-d.x2351
                                     '(pair (free-id . 1) pair (any) each any)
                                     (vector 'quote 'list 'cons))))
                                :-d.x2349)))
                           (syntax-dispatch
                             :-d.x2349
                             '(pair (free-id . 0) pair (atom) atom)
                             (vector 'quote 'list 'cons))))
                        :-d.x2347)))
                   (syntax-dispatch
                     :-d.x2347
                     '(pair (free-id . 0) pair (pair (any) each any) atom)
                     (vector 'quote 'list 'cons))))
                :-d.x2345)))
           (:-d.gen-vector2311
             (lambda (:-d.x2374)
               ((lambda (:-d.x2375)
                  ((lambda (:-d.y2376)
                     (if :-d.y2376
                       ((lambda (:-d.x2384)
                          (list 'quote (list->vector :-d.x2384)))
                        (car :-d.y2376))
                       ((lambda (:-d.x2377)
                          ((lambda (:-d.y2378)
                             (if :-d.y2378
                               ((lambda (:-d.x2382) (cons 'vector :-d.x2382))
                                (car :-d.y2378))
                               ((lambda (:-d.x2379)
                                  ((lambda (:-d.y2380)
                                     (if :-d.y2380
                                       ((lambda (:-d.x2381)
                                          (list 'list->vector :-d.x2381))
                                        (car :-d.y2380))
                                       (syntax-error :-d.x2379)))
                                   (syntax-dispatch
                                     :-d.x2379
                                     '(any)
                                     (vector 'quote 'list))))
                                :-d.x2377)))
                           (syntax-dispatch
                             :-d.x2377
                             '(pair (free-id . 1) each any)
                             (vector 'quote 'list))))
                        :-d.x2375)))
                   (syntax-dispatch
                     :-d.x2375
                     '(pair (free-id . 0) pair (each any) atom)
                     (vector 'quote 'list))))
                :-d.x2374)))
           (:-d.gen2312
             (lambda (:-d.p2386 :-d.lev2387)
               ((lambda (:-d.x2388)
                  ((lambda (:-d.y2389)
                     (if :-d.y2389
                       ((lambda (:-d.p2408)
                          (if (= :-d.lev2387 '0)
                            :-d.p2408
                            (:-d.gen-cons2309
                              (list 'quote 'unquote)
                              (:-d.gen2312
                                (list :-d.p2408)
                                (- :-d.lev2387 '1)))))
                        (car :-d.y2389))
                       ((lambda (:-d.x2390)
                          ((lambda (:-d.y2391)
                             (if :-d.y2391
                               ((lambda (:-d.p2406 :-d.q2407)
                                  (if (= :-d.lev2387 '0)
                                    (:-d.gen-append2310
                                      :-d.p2406
                                      (:-d.gen2312 :-d.q2407 :-d.lev2387))
                                    (:-d.gen-cons2309
                                      (:-d.gen-cons2309
                                        (list 'quote 'unquote-splicing)
                                        (:-d.gen2312
                                          :-d.p2406
                                          (- :-d.lev2387 '1)))
                                      (:-d.gen2312 :-d.q2407 :-d.lev2387))))
                                (car :-d.y2391)
                                (cadr :-d.y2391))
                               ((lambda (:-d.x2392)
                                  ((lambda (:-d.y2393)
                                     (if :-d.y2393
                                       ((lambda (:-d.p2405)
                                          (:-d.gen-cons2309
                                            (list 'quote 'quasiquote)
                                            (:-d.gen2312
                                              (list :-d.p2405)
                                              (+ :-d.lev2387 '1))))
                                        (car :-d.y2393))
                                       ((lambda (:-d.x2394)
                                          ((lambda (:-d.y2395)
                                             (if :-d.y2395
                                               ((lambda (:-d.p2403 :-d.q2404)
                                                  (:-d.gen-cons2309
                                                    (:-d.gen2312
                                                      :-d.p2403
                                                      :-d.lev2387)
                                                    (:-d.gen2312
                                                      :-d.q2404
                                                      :-d.lev2387)))
                                                (car :-d.y2395)
                                                (cadr :-d.y2395))
                                               ((lambda (:-d.x2396)
                                                  ((lambda (:-d.y2397)
                                                     (if :-d.y2397
                                                       ((lambda (:-d.x2401)
                                                          (:-d.gen-vector2311
                                                            (:-d.gen2312
                                                              :-d.x2401
                                                              :-d.lev2387)))
                                                        (car :-d.y2397))
                                                       ((lambda (:-d.x2398)
                                                          ((lambda (:-d.y2399)
                                                             (if :-d.y2399
                                                               ((lambda (:-d.p2400)
                                                                  (list 'quote
                                                                        :-d.p2400))
                                                                (car :-d.y2399))
                                                               (syntax-error
                                                                 :-d.x2398)))
                                                           (syntax-dispatch
                                                             :-d.x2398
                                                             '(any)
                                                             (vector 'unquote
                                                                     'unquote-splicing
                                                                     'quasiquote))))
                                                        :-d.x2396)))
                                                   (syntax-dispatch
                                                     :-d.x2396
                                                     '(vector each any)
                                                     (vector 'unquote
                                                             'unquote-splicing
                                                             'quasiquote))))
                                                :-d.x2394)))
                                           (syntax-dispatch
                                             :-d.x2394
                                             '(pair (any) any)
                                             (vector 'unquote
                                                     'unquote-splicing
                                                     'quasiquote))))
                                        :-d.x2392)))
                                   (syntax-dispatch
                                     :-d.x2392
                                     '(pair (free-id . 2) pair (any) atom)
                                     (vector 'unquote
                                             'unquote-splicing
                                             'quasiquote))))
                                :-d.x2390)))
                           (syntax-dispatch
                             :-d.x2390
                             '(pair (pair (free-id . 1) pair (any) atom) any)
                             (vector 'unquote 'unquote-splicing 'quasiquote))))
                        :-d.x2388)))
                   (syntax-dispatch
                     :-d.x2388
                     '(pair (free-id . 0) pair (any) atom)
                     (vector 'unquote 'unquote-splicing 'quasiquote))))
                :-d.p2386))))
    (lambda (:-d.x2313)
      ((lambda (:-d.x2314)
         ((lambda (:-d.y2315)
            (if :-d.y2315
              ((lambda (:-d.-2316 :-d.e2317) (:-d.gen2312 :-d.e2317 '0))
               (car :-d.y2315)
               (cadr :-d.y2315))
              (syntax-error :-d.x2314)))
          (syntax-dispatch :-d.x2314 '(pair (any) pair (any) atom) '#())))
       :-d.x2313))))

;------------------------------------------------------------------------------

(##namespace ("")) ; so file inclusion works better
